#!/usr/bin/env python3
"""Test typeshed's third party stubs using stubtest."""

from __future__ import annotations

import argparse
import os
import re
import subprocess
import sys
import tempfile
from pathlib import Path
from shutil import rmtree
from textwrap import dedent
from time import time
from typing import NoReturn

from ts_utils.metadata import NoSuchStubError, get_recursive_requirements, read_metadata
from ts_utils.mypy import mypy_configuration_from_distribution, temporary_mypy_config_file
from ts_utils.paths import STUBS_PATH, allowlists_path, tests_path
from ts_utils.utils import (
    PYTHON_VERSION,
    allowlist_stubtest_arguments,
    colored,
    get_mypy_req,
    print_divider,
    print_error,
    print_info,
    print_success_msg,
    print_time,
    print_warning,
)


def run_stubtest(dist: Path, *, verbose: bool = False, ci_platforms_only: bool = False, keep_tmp_dir: bool = False) -> bool:
    """Run stubtest for a single distribution."""

    dist_name = dist.name
    metadata = read_metadata(dist_name)
    print(f"{dist_name}... ", end="", flush=True)

    t = time()

    stubtest_settings = metadata.stubtest_settings
    if stubtest_settings.skip:
        print(colored("skipping (skip = true)", "yellow"))
        return True

    if stubtest_settings.supported_platforms is not None and sys.platform not in stubtest_settings.supported_platforms:
        print(colored("skipping (platform not supported)", "yellow"))
        return True

    if ci_platforms_only and sys.platform not in stubtest_settings.ci_platforms:
        print(colored("skipping (platform skipped in CI)", "yellow"))
        return True

    if not metadata.requires_python.contains(PYTHON_VERSION):
        print(colored(f"skipping (requires Python {metadata.requires_python})", "yellow"))
        return True

    tmp = tempfile.mkdtemp(prefix="stubtest-")  # TODO: Python 3.12: Use TemporaryDirectory
    venv_dir = Path(tmp)
    try:
        try:
            subprocess.run(["uv", "venv", venv_dir, "--seed"], capture_output=True, check=True)
        except subprocess.CalledProcessError as e:
            print_command_failure("Failed to create a virtualenv (likely a bug in uv?)", e)
            return False
        if sys.platform == "win32":
            pip_exe = str(venv_dir / "Scripts" / "pip.exe")
            python_exe = str(venv_dir / "Scripts" / "python.exe")
        else:
            pip_exe = str(venv_dir / "bin" / "pip")
            python_exe = str(venv_dir / "bin" / "python")
        dist_extras = ", ".join(stubtest_settings.extras)
        dist_req = f"{dist_name}[{dist_extras}]{metadata.version_spec}"

        requirements = get_recursive_requirements(dist_name)

        # We need stubtest to be able to import the package, so install mypy into the venv
        # Hopefully mypy continues to not need too many dependencies
        # TODO: Maybe find a way to cache these in CI
        dists_to_install = [dist_req, get_mypy_req()]
        # Internal requirements are added to MYPYPATH
        dists_to_install.extend(str(r) for r in requirements.external_pkgs)
        dists_to_install.extend(stubtest_settings.stubtest_requirements)

        # Since the "gdb" Python package is available only inside GDB, it is not
        # possible to install it through pip, so stub tests cannot install it.
        if dist_name == "gdb":
            dists_to_install[:] = dists_to_install[1:]

        pip_cmd = [pip_exe, "install", *dists_to_install]
        try:
            subprocess.run(pip_cmd, check=True, capture_output=True)
        except subprocess.CalledProcessError as e:
            print_command_failure("Failed to install", e)
            return False

        mypy_configuration = mypy_configuration_from_distribution(dist_name)
        with temporary_mypy_config_file(mypy_configuration, stubtest_settings) as temp:
            ignore_missing_stub = ["--ignore-missing-stub"] if stubtest_settings.ignore_missing_stub else []
            packages_to_check = [d.name for d in dist.iterdir() if d.is_dir() and d.name.isidentifier()]
            modules_to_check = [d.stem for d in dist.iterdir() if d.is_file() and d.suffix == ".pyi"]
            stubtest_cmd = [
                python_exe,
                "-m",
                "mypy.stubtest",
                "--mypy-config-file",
                temp.name,
                "--show-traceback",
                # Use --custom-typeshed-dir in case we make linked changes to stdlib or _typeshed
                "--custom-typeshed-dir",
                str(dist.parent.parent),
                *ignore_missing_stub,
                *packages_to_check,
                *modules_to_check,
                *allowlist_stubtest_arguments(dist_name),
            ]

            stubs_dir = dist.parent
            mypypath_items = [str(dist)] + [str(stubs_dir / pkg.name) for pkg in requirements.typeshed_pkgs]
            mypypath = os.pathsep.join(mypypath_items)
            # For packages that need a display, we need to pass at least $DISPLAY
            # to stubtest. $DISPLAY is set by xvfb-run in CI.
            #
            # It seems that some other environment variables are needed too,
            # because the CI fails if we pass only os.environ["DISPLAY"]. I didn't
            # "bisect" to see which variables are actually needed.
            stubtest_env = os.environ | {
                "MYPYPATH": mypypath,
                "MYPY_FORCE_COLOR": "1",
                # Prevent stubtest crash due to special unicode character
                # https://github.com/python/mypy/issues/19071
                "PYTHONUTF8": "1",
            }

            # Perform some black magic in order to run stubtest inside uWSGI
            if dist_name == "uWSGI":
                if not setup_uwsgi_stubtest_command(dist, venv_dir, stubtest_cmd):
                    return False

            if dist_name == "gdb":
                if not setup_gdb_stubtest_command(venv_dir, stubtest_cmd):
                    return False

            try:
                subprocess.run(stubtest_cmd, env=stubtest_env, check=True, capture_output=True)
            except subprocess.CalledProcessError as e:
                print_time(time() - t)
                print_error("fail")

                print_divider()
                print("Commands run:")
                print_commands(pip_cmd, stubtest_cmd, mypypath)

                print_divider()
                print("Command output:\n")
                print_command_output(e)

                print_divider()
                print("Python version: ", end="", flush=True)
                ret = subprocess.run([sys.executable, "-VV"], capture_output=True, check=False)
                print_command_output(ret)

                print("\nRan with the following environment:")
                ret = subprocess.run([pip_exe, "freeze", "--all"], capture_output=True, check=False)
                print_command_output(ret)
                if keep_tmp_dir:
                    print("Path to virtual environment:", venv_dir, flush=True)

                print_divider()
                main_allowlist_path = allowlists_path(dist_name) / "stubtest_allowlist.txt"
                if main_allowlist_path.exists():
                    print(f'To fix "unused allowlist" errors, remove the corresponding entries from {main_allowlist_path}')
                    print()
                else:
                    print(f"Re-running stubtest with --generate-allowlist.\nAdd the following to {main_allowlist_path}:")
                    ret = subprocess.run(
                        [*stubtest_cmd, "--generate-allowlist"], env=stubtest_env, capture_output=True, check=False
                    )
                    print_command_output(ret)

                print_divider()
                print(f"Upstream repository: {metadata.upstream_repository}")
                print(f"Typeshed source code: https://github.com/python/typeshed/tree/main/stubs/{dist.name}")

                print_divider()

                return False
            else:
                print_time(time() - t)
                print_success_msg()

                if sys.platform not in stubtest_settings.ci_platforms:
                    print_warning(f"Note: {dist_name} is not currently tested on {sys.platform} in typeshed's CI")

                if keep_tmp_dir:
                    print_info(f"Virtual environment kept at: {venv_dir}")

    finally:
        if not keep_tmp_dir:
            rmtree(venv_dir)

    if verbose:
        print_commands(pip_cmd, stubtest_cmd, mypypath)

    return True


def setup_gdb_stubtest_command(venv_dir: Path, stubtest_cmd: list[str]) -> bool:
    """
    Use wrapper scripts to run stubtest inside gdb.
    The wrapper script is used to pass the arguments to the gdb script.
    """
    if sys.platform == "win32":
        print_error("gdb is not supported on Windows")
        return False

    if not gdb_version_check():
        return False

    gdb_script = venv_dir / "gdb_stubtest.py"
    wrapper_script = venv_dir / "gdb_wrapper.py"
    gdb_script_contents = dedent(
        f"""
        import json
        import os
        import site
        import sys
        import traceback

        from glob import glob

        # Add the venv site-packages to sys.path. gdb doesn't use the virtual environment.
        # Taken from https://github.com/pwndbg/pwndbg/blob/83d8d95b576b749e888f533ce927ad5a77fb957b/gdbinit.py#L37
        site_pkgs_path = glob(os.path.join({str(venv_dir)!r}, "lib/*/site-packages"))[0]
        site.addsitedir(site_pkgs_path)

        exit_code = 1
        try:
            # gdb wraps stdout and stderr without a .fileno
            # colored output in mypy tries to access .fileno()
            sys.stdout.fileno = sys.__stdout__.fileno
            sys.stderr.fileno = sys.__stderr__.fileno

            from mypy.stubtest import main

            sys.argv = json.loads(os.environ.get("STUBTEST_ARGS"))
            exit_code = main()
        except Exception:
            traceback.print_exc()
        finally:
            gdb.execute(f"quit {{exit_code}}")
        """
    )
    gdb_script.write_text(gdb_script_contents)

    wrapper_script_contents = dedent(
        f"""
        import json
        import os
        import subprocess
        import sys

        stubtest_env = os.environ | {{"STUBTEST_ARGS": json.dumps(sys.argv)}}
        # With LD_LIBRARY_PATH set, some GitHub action runners look in the wrong
        # location for gdb, causing stubtest to fail.
        stubtest_env.pop("LD_LIBRARY_PATH", None)
        gdb_cmd = [
            "gdb",
            "--quiet",
            "--nx",
            "--batch",
            "--command",
            {str(gdb_script)!r},
        ]
        r = subprocess.run(gdb_cmd, env=stubtest_env)
        sys.exit(r.returncode)
        """
    )
    wrapper_script.write_text(wrapper_script_contents)

    # replace "-m mypy.stubtest" in stubtest_cmd with the path to our wrapper script
    assert stubtest_cmd[1:3] == ["-m", "mypy.stubtest"]
    stubtest_cmd[1:3] = [str(wrapper_script)]
    return True


def gdb_version_check() -> bool:
    try:
        gdb_version_output = subprocess.check_output(["gdb", "--version"], text=True, stderr=subprocess.STDOUT)
    except FileNotFoundError:
        print_error("gdb is not installed")
        return False
    if "Python scripting is not supported in this copy of GDB" in gdb_version_output:
        print_error("Python scripting is not supported in this copy of GDB")
        return False
    m = re.search(r"GNU gdb\s+.*?(\d+\.\d+(\.[\da-z-]+)+)", gdb_version_output)
    if m is None:
        print_error("Failed to determine gdb version:\n" + gdb_version_output)
        return False
    gdb_version = m.group(1)
    print(f"({gdb_version}) ", end="", flush=True)
    return True


def setup_uwsgi_stubtest_command(dist: Path, venv_dir: Path, stubtest_cmd: list[str]) -> bool:
    """Perform some black magic in order to run stubtest inside uWSGI.

    We have to write the exit code from stubtest to a surrogate file
    because uwsgi --pyrun does not exit with the exitcode from the
    python script. We have a second wrapper script that passed the
    arguments along to the uWSGI script and retrieves the exit code
    from the file, so it behaves like running stubtest normally would.

    Both generated wrapper scripts are created inside `venv_dir`,
    which itself is a subdirectory inside a temporary directory,
    so both scripts will be cleaned up after this function
    has been executed.
    """
    uwsgi_ini = tests_path(dist.name) / "uwsgi.ini"

    if sys.platform == "win32":
        print_error("uWSGI is not supported on Windows")
        return False

    uwsgi_script = venv_dir / "uwsgi_stubtest.py"
    wrapper_script = venv_dir / "uwsgi_wrapper.py"
    exit_code_surrogate = venv_dir / "exit_code"
    uwsgi_script_contents = dedent(
        f"""
        import json
        import os
        import sys
        from mypy.stubtest import main

        sys.argv = json.loads(os.environ.get("STUBTEST_ARGS"))
        exit_code = main()
        with open("{exit_code_surrogate}", mode="w") as fp:
            fp.write(str(exit_code))
        sys.exit(exit_code)
        """
    )
    uwsgi_script.write_text(uwsgi_script_contents)

    uwsgi_exe = venv_dir / "bin" / "uwsgi"

    # It would be nice to reliably separate uWSGI output from
    # the stubtest output, on linux it appears that stubtest
    # will always go to stdout and uWSGI to stderr, but on
    # MacOS they both go to stderr, for now we deal with the
    # bit of extra spam
    wrapper_script_contents = dedent(
        f"""
        import json
        import os
        import subprocess
        import sys

        stubtest_env = os.environ | {{"STUBTEST_ARGS": json.dumps(sys.argv)}}
        uwsgi_cmd = [
            "{uwsgi_exe}",
            "--ini",
            "{uwsgi_ini}",
            "--spooler",
            "{venv_dir}",
            "--pyrun",
            "{uwsgi_script}",
        ]
        subprocess.run(uwsgi_cmd, env=stubtest_env)
        with open("{exit_code_surrogate}", mode="r") as fp:
            sys.exit(int(fp.read()))
        """
    )
    wrapper_script.write_text(wrapper_script_contents)

    # replace "-m mypy.stubtest" in stubtest_cmd with the path to our wrapper script
    assert stubtest_cmd[1:3] == ["-m", "mypy.stubtest"]
    stubtest_cmd[1:3] = [str(wrapper_script)]
    return True


def print_commands(pip_cmd: list[str], stubtest_cmd: list[str], mypypath: str) -> None:
    print()
    print(" ".join(pip_cmd))
    print(f"MYPYPATH={mypypath}", " ".join(stubtest_cmd))


def print_command_failure(message: str, e: subprocess.CalledProcessError) -> None:
    print_error("fail")
    print()
    print(message)
    print_command_output(e)


def print_command_output(e: subprocess.CalledProcessError | subprocess.CompletedProcess[bytes]) -> None:
    print(e.stdout.decode(), end="")
    print(e.stderr.decode(), end="")


def main() -> NoReturn:
    parser = argparse.ArgumentParser()
    parser.add_argument("-v", "--verbose", action="store_true", help="verbose output")
    parser.add_argument("--num-shards", type=int, default=1)
    parser.add_argument("--shard-index", type=int, default=0)
    parser.add_argument(
        "--ci-platforms-only",
        action="store_true",
        help="skip the test if the current platform is not specified in METADATA.toml/tool.stubtest.ci-platforms",
    )
    parser.add_argument("--keep-tmp-dir", action="store_true", help="keep the temporary virtualenv")
    parser.add_argument("dists", metavar="DISTRIBUTION", type=str, nargs=argparse.ZERO_OR_MORE)
    args = parser.parse_args()

    if len(args.dists) == 0:
        dists = sorted(STUBS_PATH.iterdir())
    else:
        dists = [STUBS_PATH / d for d in args.dists]

    result = 0
    for i, dist in enumerate(dists):
        if i % args.num_shards != args.shard_index:
            continue
        try:
            if not run_stubtest(
                dist, verbose=args.verbose, ci_platforms_only=args.ci_platforms_only, keep_tmp_dir=args.keep_tmp_dir
            ):
                result = 1
        except NoSuchStubError as e:
            parser.error(str(e))
    sys.exit(result)


if __name__ == "__main__":
    try:
        main()
    except KeyboardInterrupt:
        pass



import sys
from _typeshed import Incomplete
from collections.abc import Callable, ItemsView, Iterable, Iterator, KeysView, ValuesView
from dataclasses import dataclass
from types import TracebackType
from typing_extensions import Self

__all__ = ["Config"]

# TODO: Our pyright test doesn't understand `requires_python` in METADATA.toml
# https://github.com/python/typeshed/issues/14025
if sys.version_info >= (3, 10):
    @dataclass(init=False, eq=False, slots=True, kw_only=True, match_args=False)
    class Config:
        def __init_subclass__(cls, strict: bool = True) -> None: ...
        def __new__(cls, **kwargs) -> Self: ...
        def __dir__(self) -> Iterable[str]: ...
        def __setattr__(self, name: str, value) -> None: ...
        def __delattr__(self, name: str) -> None: ...
        def __contains__(self, key: object) -> bool: ...
        def __iter__(self) -> Iterator[str]: ...
        def __len__(self) -> int: ...
        def __reversed__(self) -> Iterator[str]: ...
        def __getitem__(self, key: str): ...
        def __setitem__(self, key: str, value) -> None: ...
        def __delitem__(self, key: str) -> None: ...
        def get(self, key: str, default=None): ...
        def items(self) -> ItemsView[str, Incomplete]: ...
        def keys(self) -> KeysView[str]: ...
        def values(self) -> ValuesView[Incomplete]: ...
        def __reduce__(self) -> tuple[Callable[..., Self], tuple[type[Self], dict[Incomplete, Incomplete]]]: ...
        def __call__(self, **kwargs) -> Self: ...
        def __enter__(self) -> Self: ...
        def __exit__(
            self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None
        ) -> None: ...

else:
    @dataclass(init=False, eq=False)
    class Config:
        def __init_subclass__(cls, strict: bool = True) -> None: ...
        def __new__(cls, **kwargs) -> Self: ...
        def __dir__(self) -> Iterable[str]: ...
        def __setattr__(self, name: str, value) -> None: ...
        def __delattr__(self, name: str) -> None: ...
        def __contains__(self, key: object) -> bool: ...
        def __iter__(self) -> Iterator[str]: ...
        def __len__(self) -> int: ...
        def __reversed__(self) -> Iterator[str]: ...
        def __getitem__(self, key: str): ...
        def __setitem__(self, key: str, value) -> None: ...
        def __delitem__(self, key: str) -> None: ...
        def get(self, key: str, default=None): ...
        def items(self) -> ItemsView[str, Incomplete]: ...
        def keys(self) -> KeysView[str]: ...
        def values(self) -> ValuesView[Incomplete]: ...
        def __reduce__(self) -> tuple[Callable[..., Self], tuple[type[Self], dict[Incomplete, Incomplete]]]: ...
        def __call__(self, **kwargs) -> Self: ...
        def __enter__(self) -> Self: ...
        def __exit__(
            self, exc_type: type[BaseException] | None, exc_value: BaseException | None, traceback: TracebackType | None
        ) -> None: ...

class NetworkXConfig(Config):
    backend_priority: list[str]
    backends: Config
    cache_converted_graphs: bool
    fallback_to_nx: bool
    warnings_to_ignore: set[str]
    def __init__(
        self,
        *,
        backend_priority: list[str],
        backends: Config,
        cache_converted_graphs: bool,
        fallback_to_nx: bool,
        warnings_to_ignore: set[str],
    ) -> None: ...
    def __new__(
        cls,
        *,
        backend_priority: list[str],
        backends: Config,
        cache_converted_graphs: bool,
        fallback_to_nx: bool,
        warnings_to_ignore: set[str],
    ) -> Self: ...

config: NetworkXConfig



from _typeshed import Incomplete

class Store:
    program: str
    exe: str | None
    environment: Incomplete
    def __init__(self, program: str, environment=None) -> None: ...
    def get(self, server): ...
    def store(self, server, username, secret): ...
    def erase(self, server) -> None: ...
    def list(self): ...



import sys
from _typeshed import ConvertibleToFloat, Incomplete, StrOrBytesPath, Unused
from collections.abc import Callable, Generator
from typing import Final, NamedTuple, TypeVar, overload
from typing_extensions import ParamSpec, TypeAlias

from PIL import Image

_P = ParamSpec("_P")
_R = TypeVar("_R")
# cv2.typing.MatLike: is an alias for `numpy.ndarray | cv2.mat_wrapper.Mat`, Mat extends ndarray.
# But can't import either, because pyscreeze does not declare them as dependencies, stub_uploader won't let it.
_MatLike: TypeAlias = Incomplete

PILLOW_VERSION: Final[tuple[int, int, int]]
RUNNING_PYTHON_2: Final = False
SCROT_EXISTS: Final[bool]
GNOMESCREENSHOT_EXISTS: Final[bool]

if sys.platform == "linux":
    RUNNING_X11: Final[bool]
    RUNNING_WAYLAND: Final[bool]

# Meant to be overridable as a setting
GRAYSCALE_DEFAULT: bool
# Meant to be overridable for backward-compatibility
USE_IMAGE_NOT_FOUND_EXCEPTION: bool

class Box(NamedTuple):
    left: int
    top: int
    width: int
    height: int

class Point(NamedTuple):
    x: int
    y: int

class RGB(NamedTuple):
    red: int
    green: int
    blue: int

class PyScreezeException(Exception): ...
class ImageNotFoundException(PyScreezeException): ...

def requiresPyGetWindow(wrappedFunction: Callable[_P, _R]) -> Callable[_P, _R]: ...

# _locateAll_opencv
@overload
def locate(
    needleImage: str | Image.Image | _MatLike,
    haystackImage: str | Image.Image | _MatLike,
    *,
    grayscale: bool | None = None,
    limit: Unused = 1,
    region: tuple[int, int, int, int] | None = None,
    step: int = 1,
    confidence: ConvertibleToFloat = 0.999,
) -> Box | None: ...

# _locateAll_pillow
@overload
def locate(
    needleImage: str | Image.Image,
    haystackImage: str | Image.Image,
    *,
    grayscale: bool | None = None,
    limit: Unused = 1,
    region: tuple[int, int, int, int] | None = None,
    step: int = 1,
    confidence: None = None,
) -> Box | None: ...

# _locateAll_opencv
@overload
def locateOnScreen(
    image: str | Image.Image | _MatLike,
    minSearchTime: float = 0,
    *,
    grayscale: bool | None = None,
    limit: Unused = 1,
    region: tuple[int, int, int, int] | None = None,
    step: int = 1,
    confidence: ConvertibleToFloat = 0.999,
) -> Box | None: ...

# _locateAll_pillow
@overload
def locateOnScreen(
    image: str | Image.Image,
    minSearchTime: float = 0,
    *,
    grayscale: bool | None = None,
    limit: Unused = 1,
    region: tuple[int, int, int, int] | None = None,
    step: int = 1,
    confidence: None = None,
) -> Box | None: ...

# _locateAll_opencv
@overload
def locateAllOnScreen(
    image: str | Image.Image | _MatLike,
    *,
    grayscale: bool | None = None,
    limit: int = 1000,
    region: tuple[int, int, int, int] | None = None,
    step: int = 1,
    confidence: ConvertibleToFloat = 0.999,
) -> Generator[Box, None, None]: ...

# _locateAll_pillow
@overload
def locateAllOnScreen(
    image: str | Image.Image,
    *,
    grayscale: bool | None = None,
    limit: int | None = None,
    region: tuple[int, int, int, int] | None = None,
    step: int = 1,
    confidence: None = None,
) -> Generator[Box, None, None]: ...

# _locateAll_opencv
@overload
def locateCenterOnScreen(
    image: str | Image.Image | _MatLike,
    *,
    minSearchTime: float = 0,
    grayscale: bool | None = None,
    limit: Unused = 1,
    region: tuple[int, int, int, int] | None = None,
    step: int = 1,
    confidence: ConvertibleToFloat = 0.999,
) -> Point | None: ...

# _locateAll_pillow
@overload
def locateCenterOnScreen(
    image: str | Image.Image,
    *,
    minSearchTime: float = 0,
    grayscale: bool | None = None,
    limit: Unused = 1,
    region: tuple[int, int, int, int] | None = None,
    step: int = 1,
    confidence: None = None,
) -> Point | None: ...
def locateOnScreenNear(image: str | Image.Image | _MatLike, x: int, y: int) -> Box: ...
def locateCenterOnScreenNear(image: str | Image.Image | _MatLike, x: int, y: int) -> Point | None: ...

# _locateAll_opencv
@overload
def locateOnWindow(
    image: str | Image.Image | _MatLike,
    title: str,
    *,
    grayscale: bool | None = None,
    limit: Unused = 1,
    step: int = 1,
    confidence: ConvertibleToFloat = 0.999,
) -> Box | None: ...

# _locateAll_pillow
@overload
def locateOnWindow(
    image: str | Image.Image,
    title: str,
    *,
    grayscale: bool | None = None,
    limit: Unused = 1,
    step: int = 1,
    confidence: None = None,
) -> Box | None: ...
def showRegionOnScreen(
    region: tuple[int, int, int, int], outlineColor: str = "red", filename: str = "_showRegionOnScreen.png"
) -> None: ...
def center(coords: tuple[int, int, int, int]) -> Point: ...
def pixelMatchesColor(
    x: int, y: int, expectedRGBColor: tuple[int, int, int] | tuple[int, int, int, int], tolerance: int = 0
) -> bool: ...
def pixel(x: int, y: int) -> tuple[int, int, int]: ...

if sys.platform == "win32":
    def screenshot(
        imageFilename: StrOrBytesPath | None = None, region: tuple[int, int, int, int] | None = None, allScreens: bool = False
    ) -> Image.Image: ...

else:
    def screenshot(
        imageFilename: StrOrBytesPath | None = None, region: tuple[int, int, int, int] | None = None
    ) -> Image.Image: ...

# _locateAll_opencv
@overload
def locateAll(
    needleImage: str | Image.Image | _MatLike,
    haystackImage: str | Image.Image | _MatLike,
    grayscale: bool | None = None,
    limit: int = 1000,
    region: tuple[int, int, int, int] | None = None,
    step: int = 1,
    confidence: ConvertibleToFloat = 0.999,
) -> Generator[Box, None, None]: ...

# _locateAll_pillow
@overload
def locateAll(
    needleImage: str | Image.Image,
    haystackImage: str | Image.Image,
    grayscale: bool | None = None,
    limit: int | None = None,
    region: tuple[int, int, int, int] | None = None,
    step: int = 1,
    confidence: None = None,
) -> Generator[Box, None, None]: ...



from _typeshed import Incomplete
from typing import IO, Final

from reportlab.graphics.renderbase import Renderer
from reportlab.graphics.shapes import Drawing
from reportlab.pdfgen.canvas import Canvas

__version__: Final[str]

def Color2Hex(c): ...
def CairoColor(c): ...
def draw(drawing: Drawing, canvas: Canvas, x: float, y: float, showBoundary=...) -> None: ...

class _PMRenderer(Renderer):
    def pop(self) -> None: ...
    def push(self, node) -> None: ...
    def applyState(self) -> None: ...
    def initState(self, x, y) -> None: ...
    def drawNode(self, node) -> None: ...
    def drawRect(self, rect) -> None: ...
    def drawLine(self, line) -> None: ...
    def drawImage(self, image) -> None: ...
    def drawCircle(self, circle) -> None: ...
    def drawPolyLine(self, polyline, _doClose: int = 0) -> None: ...
    def drawEllipse(self, ellipse) -> None: ...
    def drawPolygon(self, polygon) -> None: ...
    def drawString(self, stringObj) -> None: ...
    def drawPath(self, path): ...

BEZIER_ARC_MAGIC: float

class PMCanvas:
    ctm: Incomplete
    def __init__(
        self, w, h, dpi: int = 72, bg: int = 16777215, configPIL=None, backend=None, backendFmt: str = "RGB"
    ) -> None: ...
    def toPIL(self): ...
    def saveToFile(self, fn, fmt=None): ...
    def saveToString(self, fmt: str = "GIF"): ...
    def setFont(self, fontName, fontSize, leading=None) -> None: ...
    def __setattr__(self, name, value) -> None: ...
    def __getattr__(self, name): ...
    def fillstrokepath(self, stroke: int = 1, fill: int = 1) -> None: ...
    def bezierArcCCW(self, cx, cy, rx, ry, theta0, theta1): ...
    def addEllipsoidalArc(self, cx, cy, rx, ry, ang1, ang2) -> None: ...
    def drawCentredString(
        self, x: float, y: float, text: str, text_anchor: str = "middle", direction: str | None = None, shaping: bool = False
    ) -> None: ...
    def drawRightString(self, text: str, x: float, y: float, direction: str | None = None) -> None: ...
    def drawString(
        self,
        x: float,
        y: float,
        text: str,
        _fontInfo=None,
        text_anchor: str = "left",
        direction: str | None = None,
        shaping: bool = False,
    ) -> None: ...
    def line(self, x1, y1, x2, y2) -> None: ...
    def rect(self, x, y, width, height, stroke: int = 1, fill: int = 1) -> None: ...
    def roundRect(self, x, y, width, height, rx, ry) -> None: ...
    def circle(self, cx, cy, r) -> None: ...
    def ellipse(self, cx, cy, rx, ry) -> None: ...
    def saveState(self) -> None: ...
    fillColor: Incomplete
    fillOpacity: Incomplete
    def setFillColor(self, aColor) -> None: ...
    strokeColor: Incomplete
    strokeOpacity: Incomplete
    def setStrokeColor(self, aColor) -> None: ...
    restoreState = saveState
    lineCap: Incomplete
    def setLineCap(self, cap) -> None: ...
    lineJoin: Incomplete
    def setLineJoin(self, join) -> None: ...
    strokeWidth: Incomplete
    def setLineWidth(self, width) -> None: ...
    def stringWidth(self, text, fontName=None, fontSize=None): ...

def drawToPMCanvas(
    d: Drawing,
    dpi: float = 72,
    bg: int = 0xFFFFFF,
    configPIL=None,
    showBoundary=...,
    backend="rlPyCairo",
    backendFmt: str = "RGB",
): ...
def drawToPIL(
    d: Drawing,
    dpi: float = 72,
    bg: int = 0xFFFFFF,
    configPIL=None,
    showBoundary=...,
    backend="rlPyCairo",
    backendFmt: str = "RGB",
): ...
def drawToPILP(
    d: Drawing,
    dpi: float = 72,
    bg: int = 0xFFFFFF,
    configPIL=None,
    showBoundary=...,
    backend="rlPyCairo",
    backendFmt: str = "RGB",
): ...
def drawToFile(
    d: Drawing,
    fn: str | IO[bytes],
    fmt: str = "GIF",
    dpi: float = 72,
    bg: int = 0xFFFFFF,
    configPIL=None,
    showBoundary=...,
    backend="rlPyCairo",
    backendFmt: str = "RGB",
) -> None: ...
def drawToString(
    d: Drawing,
    fmt: str = "GIF",
    dpi: float = 72,
    bg: int = 0xFFFFFF,
    configPIL=None,
    showBoundary=...,
    backend="rlPyCairo",
    backendFmt: str = "RGB",
) -> str: ...

save = drawToFile

def test(outDir: str = "pmout", shout: bool = False): ...



from _typeshed import Incomplete
from collections.abc import Callable, Sequence
from logging import Logger

log: Logger

class BaseEndpoint:
    def __init__(self) -> None: ...
    @property
    def valid_request_methods(self) -> Sequence[str] | None: ...
    @valid_request_methods.setter
    def valid_request_methods(self, valid_request_methods: Sequence[str] | None) -> None: ...
    @property
    def available(self) -> bool: ...
    @available.setter
    def available(self, available: bool) -> None: ...
    @property
    def catch_errors(self) -> bool: ...
    @catch_errors.setter
    def catch_errors(self, catch_errors: bool) -> None: ...

def catch_errors_and_unavailability(
    f: Callable[..., tuple[dict[str, Incomplete], str, int]],
) -> Callable[..., tuple[dict[str, Incomplete], str, int]]: ...



from .authorization_server import JWTAuthenticationRequest as JWTAuthenticationRequest
from .discovery import AuthorizationServerMetadata as AuthorizationServerMetadata
from .registration import ClientMetadataClaims as ClientMetadataClaims

__all__ = ["AuthorizationServerMetadata", "JWTAuthenticationRequest", "ClientMetadataClaims"]



# from fontTools.misc.loggingTools
from abc import ABCMeta, abstractmethod
from collections.abc import Mapping
from logging import Logger
from typing import Any, Protocol
from typing_extensions import TypeAlias

# from fonttools.ttLib.ttGlyphSet
class _TTGlyph(Protocol):
    def __init__(self, glyphSet: _TTGlyphSet, glyphName: str) -> None: ...
    def draw(self, pen) -> None: ...
    def drawPoints(self, pen) -> None: ...

_TTGlyphSet: TypeAlias = Mapping[str, _TTGlyph]  # Simplified for our needs

# fonttools.ttLib.TTFont
_TTFont: TypeAlias = Any  # noqa: Y047

# from fontTools.misc.loggingTools

class LogMixin:
    @property
    def log(self) -> Logger: ...

# from fontTools.pens.basePen
class AbstractPen:
    @abstractmethod
    def moveTo(self, pt: tuple[float, float]) -> None: ...
    @abstractmethod
    def lineTo(self, pt: tuple[float, float]) -> None: ...
    @abstractmethod
    def curveTo(self, *points: tuple[float, float]) -> None: ...
    @abstractmethod
    def qCurveTo(self, *points: tuple[float, float]) -> None: ...
    def closePath(self) -> None: ...
    def endPath(self) -> None: ...
    @abstractmethod
    def addComponent(self, glyphName: str, transformation: tuple[float, float, float, float, float, float]) -> None: ...

class LoggingPen(LogMixin, AbstractPen, metaclass=ABCMeta): ...

class DecomposingPen(LoggingPen, metaclass=ABCMeta):
    skipMissingComponents: bool
    glyphSet: _TTGlyphSet | None
    def __init__(self, glyphSet: _TTGlyphSet | None) -> None: ...
    def addComponent(self, glyphName: str, transformation: tuple[float, float, float, float, float, float]) -> None: ...

class BasePen(DecomposingPen):
    def __init__(self, glyphSet: _TTGlyphSet | None = ...) -> None: ...
    def closePath(self) -> None: ...
    def endPath(self) -> None: ...
    def moveTo(self, pt: tuple[float, float]) -> None: ...
    def lineTo(self, pt: tuple[float, float]) -> None: ...
    def curveTo(self, *points: tuple[float, float]) -> None: ...
    def qCurveTo(self, *points: tuple[float, float]) -> None: ...



from logging import Logger
from typing import Any

from oauthlib.oauth1 import Client
from requests.auth import AuthBase

CONTENT_TYPE_FORM_URLENCODED: str
CONTENT_TYPE_MULTI_PART: str
log: Logger

class OAuth1(AuthBase):
    client_class: type[Client]
    client: Client
    force_include_body: bool
    def __init__(
        self,
        client_key,
        client_secret=None,
        resource_owner_key=None,
        resource_owner_secret=None,
        callback_uri=None,
        signature_method="HMAC-SHA1",
        signature_type="AUTH_HEADER",
        rsa_key=None,
        verifier=None,
        decoding: str | None = "utf-8",
        client_class: type[Client] | None = None,
        force_include_body: bool = False,
        *,
        realm=None,
        encoding: str = "utf-8",
        nonce=None,
        timestamp=None,
        **kwargs: Any,  # passed to client_class's __init__
    ) -> None: ...



from oauthlib.common import Request
from oauthlib.oauth2 import RequestValidator as OAuth2RequestValidator

class RequestValidator(OAuth2RequestValidator):
    def client_authentication_required(self, request: Request, *args, **kwargs) -> bool: ...



from collections.abc import Callable, Sequence
from typing import Any, Final, Generic, NamedTuple, TypeVar, overload

__all__: Final = ("fifo_cache", "lfu_cache", "lru_cache", "rr_cache", "ttl_cache")

_T = TypeVar("_T")
_R = TypeVar("_R")

class _CacheInfo(NamedTuple):
    hits: int
    misses: int
    maxsize: int | None
    currsize: int

class _cachetools_cache_wrapper(Generic[_R]):
    __wrapped__: Callable[..., _R]
    def __call__(self, /, *args: Any, **kwargs: Any) -> _R: ...
    def cache_info(self) -> _CacheInfo: ...
    def cache_clear(self) -> None: ...
    def cache_parameters(self) -> dict[str, Any]: ...

@overload
def fifo_cache(
    maxsize: int | None = 128, typed: bool = False
) -> Callable[[Callable[..., _R]], _cachetools_cache_wrapper[_R]]: ...
@overload
def fifo_cache(maxsize: Callable[..., _R], typed: bool = False) -> _cachetools_cache_wrapper[_R]: ...
@overload
def lfu_cache(maxsize: int | None = 128, typed: bool = False) -> Callable[[Callable[..., _R]], _cachetools_cache_wrapper[_R]]: ...
@overload
def lfu_cache(maxsize: Callable[..., _R], typed: bool = False) -> _cachetools_cache_wrapper[_R]: ...
@overload
def lru_cache(maxsize: int | None = 128, typed: bool = False) -> Callable[[Callable[..., _R]], _cachetools_cache_wrapper[_R]]: ...
@overload
def lru_cache(maxsize: Callable[..., _R], typed: bool = False) -> _cachetools_cache_wrapper[_R]: ...
@overload
def rr_cache(
    maxsize: int | None = 128, choice: Callable[[Sequence[_T]], _T] | None = ..., typed: bool = False
) -> Callable[[Callable[..., _R]], _cachetools_cache_wrapper[_R]]: ...
@overload
def rr_cache(
    maxsize: Callable[..., _R], choice: Callable[[Sequence[_T]], _T] | None = ..., typed: bool = False
) -> _cachetools_cache_wrapper[_R]: ...
@overload
def ttl_cache(
    maxsize: int | None = 128, ttl: float = 600, timer: Callable[[], float] = ..., typed: bool = False
) -> Callable[[Callable[..., _R]], _cachetools_cache_wrapper[_R]]: ...
@overload
def ttl_cache(
    maxsize: Callable[..., _R], ttl: float = 600, timer: Callable[[], float] = ..., typed: bool = False
) -> _cachetools_cache_wrapper[_R]: ...



from _typeshed.wsgi import StartResponse, WSGIApplication, WSGIEnvironment
from collections.abc import Iterable, Mapping
from typing import Any

from sassutils.builder import Manifest

class SassMiddleware:
    app: WSGIApplication
    manifests: dict[str, Manifest]
    error_status: str
    package_dir: Mapping[str, str]
    paths: list[tuple[str, str, Manifest]]
    def __init__(
        self,
        app: WSGIApplication,
        manifests: Mapping[str, Manifest | tuple[Any, ...] | Mapping[str, Any] | str] | None,
        package_dir: Mapping[str, str] = {},
        error_status: str = "200 OK",
    ) -> None: ...
    def __call__(self, environ: WSGIEnvironment, start_response: StartResponse) -> Iterable[bytes]: ...
    @staticmethod
    def quote_css_string(s: str) -> str: ...

__all__ = ("SassMiddleware",)



# https://pyinstaller.org/en/stable/hooks.html#the-pre-safe-import-module-psim-api-method

# The documentation explicitly mentions that "Normally you do not need to know about the module-graph."
# However, some PyiModuleGraph typed class attributes are still documented as existing in imphookapi.
from _typeshed import Incomplete, StrPath, SupportsKeysAndGetItem
from collections.abc import Iterable
from typing_extensions import TypeAlias

from PyInstaller.lib.modulegraph.modulegraph import Alias, Node

_LazyNode: TypeAlias = Iterable[Node] | Iterable[str] | Alias | None
# from altgraph.Graph import Graph
_Graph: TypeAlias = Incomplete

class PyiModuleGraph:  # incomplete
    def __init__(
        self,
        pyi_homepath: str,
        user_hook_dirs: Iterable[StrPath] = (),
        excludes: Iterable[str] = (),
        *,
        path: Iterable[str] | None = None,
        replace_paths: Iterable[tuple[StrPath, StrPath]] = ...,
        implies: SupportsKeysAndGetItem[str, _LazyNode] | Iterable[tuple[str, _LazyNode]] = ...,
        graph: _Graph | None = None,
        debug: bool = False,
    ) -> None: ...



from _typeshed import Incomplete
from typing import Final

from reportlab.graphics.charts.piecharts import AbstractPieChart, WedgeProperties
from reportlab.lib.attrmap import *

__version__: Final[str]

class SectorProperties(WedgeProperties): ...

class Doughnut(AbstractPieChart):
    x: int
    y: int
    width: int
    height: int
    data: Incomplete
    labels: Incomplete
    startAngle: int
    direction: str
    simpleLabels: int
    checkLabelOverlap: int
    sideLabels: int
    innerRadiusFraction: Incomplete
    slices: Incomplete
    angleRange: int
    def __init__(self, *, angleRange: int = 360, **kwds) -> None: ...
    def demo(self): ...
    def normalizeData(self, data=None): ...
    def makeSectors(self): ...
    def draw(self): ...

def sample1(): ...
def sample2(): ...
def sample3(): ...
def sample4(): ...



from _typeshed import Unused
from typing import TypedDict, type_check_only

@type_check_only
class _ReadMemoryResult(TypedDict):
    address: str
    data: str

def read_memory(*, memoryReference: str, offset: int = 0, count: int, **extra: Unused) -> _ReadMemoryResult: ...
def write_memory(*, memoryReference: str, offset: int = 0, data: str, **extra: Unused): ...



from collections.abc import Sequence
from re import Pattern
from typing import Protocol, type_check_only

import gdb

def register_xmethod_matcher(
    locus: gdb.Objfile | gdb.Progspace | None, matcher: XMethodMatcher, replace: bool = False
) -> None: ...
@type_check_only
class _XMethod(Protocol):
    name: str
    enabled: bool

class XMethod:
    name: str
    enabled: bool

    def __init__(self, name: str) -> None: ...

class XMethodWorker:
    def get_arg_types(self) -> Sequence[gdb.Type] | gdb.Type | None: ...
    def get_result_type(self, *args: gdb.Value) -> gdb.Type: ...
    def __call__(self, *args: gdb.Value) -> object: ...

class XMethodMatcher:
    name: str
    enabled: bool
    methods: list[_XMethod] | None

    def __init__(self, name: str) -> None: ...
    def match(self, class_type: gdb.Type, method_name: str) -> XMethodWorker | Sequence[XMethodWorker]: ...

@type_check_only
class _SimpleWorkerMethod(Protocol):
    def __call__(self, *args: gdb.Value) -> object: ...

class SimpleXMethodMatcher(XMethodMatcher):
    def __init__(
        self,
        name: str,
        class_matcher: str | Pattern[str],
        method_matcher: str | Pattern[str],
        method_function: _SimpleWorkerMethod,
        *arg_types: Sequence[gdb.Type] | gdb.Type | None,
    ) -> None: ...



# Stub-only module, can't be imported at runtime.

import sys
from collections.abc import Collection
from typing import Any, Protocol, type_check_only
from typing_extensions import TypeAlias, TypeVar

import numpy as np

_ScalarT = TypeVar("_ScalarT", bound=bool | int | float | complex | str | bytes | np.generic)
_GenericT = TypeVar("_GenericT", bound=np.generic)
_GenericT_co = TypeVar("_GenericT_co", bound=np.generic, covariant=True)
_ShapeT_co = TypeVar("_ShapeT_co", bound=tuple[int, ...], default=Any, covariant=True)

# numpy aliases
if sys.version_info >= (3, 10):
    @type_check_only
    class SupportsArray(Protocol[_GenericT_co, _ShapeT_co]):
        def __array__(self) -> np.ndarray[_ShapeT_co, np.dtype[_GenericT_co]]: ...

    ArrayLike1D: TypeAlias = Collection[_ScalarT] | SupportsArray[_GenericT, tuple[int]]
else:
    # networkx does not support Python 3.9 but pyright still runs on 3.9 in CI
    # See https://github.com/python/typeshed/issues/10722
    ArrayLike1D: TypeAlias = Collection[_ScalarT] | np.ndarray[tuple[int], np.dtype[_GenericT]]

Array1D: TypeAlias = np.ndarray[tuple[int], np.dtype[_GenericT]]
Array2D: TypeAlias = np.ndarray[tuple[int, int], np.dtype[_GenericT]]
Seed: TypeAlias = int | np.random.Generator | np.random.RandomState



#!/usr/bin/env python3

# TODO: It should be possible to specify the Python version and platform
# and limit the output to the packages that are compatible with that version
# and platform.
import sys

from ts_utils.requirements import get_external_stub_requirements

if __name__ == "__main__":
    distributions = sys.argv[1:]
    for requirement in sorted(get_external_stub_requirements(distributions), key=str):
        print(requirement)



from _typeshed import FileDescriptorOrPath
from typing import Any

def main(template: FileDescriptorOrPath, data: FileDescriptorOrPath | None = None, **kwargs: Any) -> str: ...
def cli_main() -> None: ...



import datetime
from typing import ClassVar

from ..core import WesternCalendar

class Australia(WesternCalendar):
    include_queens_birthday: ClassVar[bool]
    include_labour_day_october: ClassVar[bool]
    shift_anzac_day: ClassVar[bool]
    ANZAC_SHIFT_DAYS: ClassVar[tuple[int, ...]]
    def get_canberra_day(self, year: int) -> tuple[datetime.date, str]: ...
    def get_queens_birthday(self, year: int) -> tuple[datetime.date, str]: ...
    def get_labour_day_october(self, year: int) -> tuple[datetime.date, str]: ...
    def get_anzac_day(self, year: int) -> tuple[datetime.date, str]: ...

class AustralianCapitalTerritory(Australia):
    def get_family_community_day(self, year: int) -> tuple[datetime.date, str] | None: ...
    def get_reconciliation_day(self, year: int) -> tuple[datetime.date, str] | None: ...

class NewSouthWales(Australia): ...

class NorthernTerritory(Australia):
    def get_may_day(self, year: int) -> tuple[datetime.date, str]: ...
    def get_picnic_day(self, year: int) -> tuple[datetime.date, str]: ...

class Queensland(Australia):
    def get_labour_day_may(self, year: int) -> tuple[datetime.date, str]: ...

class SouthAustralia(Australia):
    def get_adelaides_cup(self, year: int) -> tuple[datetime.date, str]: ...
    def get_proclamation_day(self, year: int) -> tuple[datetime.date, str]: ...

class Tasmania(Australia):
    @property
    def has_recreation_day(self) -> bool: ...
    def get_eight_hours_day(self, year: int) -> tuple[datetime.date, str]: ...
    def get_recreation_day(self, year: int) -> tuple[datetime.date, str]: ...

class Hobart(Tasmania):
    def get_hobart(self, year: int) -> tuple[datetime.date, str]: ...

class Victoria(Australia):
    def get_labours_day_in_march(self, year: int) -> tuple[datetime.date, str]: ...
    def get_melbourne_cup(self, year: int) -> tuple[datetime.date, str]: ...

class WesternAustralia(Australia):
    def get_labours_day_in_march(self, year: int) -> tuple[datetime.date, str]: ...
    def get_western_australia_day(self, year: int) -> tuple[datetime.date, str]: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2016 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
==============================================================================
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class JobDef(google.protobuf.message.Message):
    """This file contains protos to be used when defining a TensorFlow
    cluster.

    EXAMPLES
    --------

    1. A single-process cluster, containing "/job:local/task:0".

       Cluster:
         job { name: 'local' tasks { key: 0 value: 'localhost:2222' } }

       Server:
         cluster { $CLUSTER } job_name: 'local' task_index: 0

    2. A two-process cluster, containing "/job:local/task:{0,1}".

       Cluster:
         job { name: 'local' tasks { key: 0 value: 'localhost:2222' }
                             tasks { key: 1 value: 'localhost:2223' } }

       Servers:
         cluster { $CLUSTER } job_name: 'local' task_index: 0
         cluster { $CLUSTER } job_name: 'local' task_index: 1

    3. A two-job cluster, containing "/job:worker/task:{0,1,2}" and
       "/job:ps/task:{0,1}".

       Cluster:
         job { name: 'worker' tasks { key: 0 value: 'worker1:2222' }
                              tasks { key: 1 value: 'worker2:2222' }
                              tasks { key: 2 value: 'worker3:2222' } }
         job { name: 'ps'     tasks { key: 0 value: 'ps0:2222' }
                              tasks { key: 1 value: 'ps1:2222' } }

       Servers:
         cluster { $CLUSTER } job_name: 'worker' task_index: 0
         cluster { $CLUSTER } job_name: 'worker' task_index: 1
         cluster { $CLUSTER } job_name: 'worker' task_index: 2
         cluster { $CLUSTER } job_name: 'ps'     task_index: 0
         cluster { $CLUSTER } job_name: 'ps'     task_index: 1

    Defines a single job in a TensorFlow cluster.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class TasksEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.int
        value: builtins.str
        def __init__(self, *, key: builtins.int | None = ..., value: builtins.str | None = ...) -> None: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    NAME_FIELD_NUMBER: builtins.int
    TASKS_FIELD_NUMBER: builtins.int
    name: builtins.str
    """The name of this job."""
    @property
    def tasks(self) -> google.protobuf.internal.containers.ScalarMap[builtins.int, builtins.str]:
        """Mapping from task ID to "hostname:port" string.

        If the `name` field contains "worker", and the `tasks` map contains a
        mapping from 7 to "example.org:2222", then the device prefix
        "/job:worker/task:7" will be assigned to "example.org:2222".

        If a job has multiple replicas, host-ports will be comma-delimited, with
        one entry for each replica.
        """

    def __init__(
        self, *, name: builtins.str | None = ..., tasks: collections.abc.Mapping[builtins.int, builtins.str] | None = ...
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["name", b"name", "tasks", b"tasks"]) -> None: ...

global___JobDef = JobDef

@typing.final
class ClusterDef(google.protobuf.message.Message):
    """Defines a TensorFlow cluster as a set of jobs."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    JOB_FIELD_NUMBER: builtins.int
    @property
    def job(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___JobDef]:
        """The jobs that comprise the cluster."""

    def __init__(self, *, job: collections.abc.Iterable[global___JobDef] | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["job", b"job"]) -> None: ...

global___ClusterDef = ClusterDef



from _typeshed import Incomplete
from typing import Final

from reportlab.graphics.charts.areas import PlotArea
from reportlab.lib.colors import black

class _BarcodeWidget(PlotArea):
    textColor = black
    barFillColor = black
    barStrokeColor: Incomplete
    barStrokeWidth: int
    x: int
    def __init__(self, _value: str = "", **kw) -> None: ...
    def rect(self, x, y, w, h, **kw) -> None: ...
    canv: Incomplete
    def draw(self): ...
    def annotate(self, x, y, text, fontName, fontSize, anchor: str = "middle") -> None: ...

class BarcodeI2of5(_BarcodeWidget):
    codeName: Final = "I2of5"
    def __init__(self, **kw) -> None: ...

class BarcodeCode128(_BarcodeWidget):
    codeName: Final = "Code128"
    def __init__(self, **kw) -> None: ...

class BarcodeStandard93(_BarcodeWidget):
    codeName: Final = "Standard93"
    def __init__(self, **kw) -> None: ...

class BarcodeExtended93(_BarcodeWidget):
    codeName: Final = "Extended93"
    def __init__(self, **kw) -> None: ...

class BarcodeStandard39(_BarcodeWidget):
    codeName: Final = "Standard39"
    def __init__(self, **kw) -> None: ...

class BarcodeExtended39(_BarcodeWidget):
    codeName: Final = "Extended39"
    def __init__(self, **kw) -> None: ...

class BarcodeMSI(_BarcodeWidget):
    codeName: Final = "MSI"
    def __init__(self, **kw) -> None: ...

class BarcodeCodabar(_BarcodeWidget):
    codeName: Final = "Codabar"
    def __init__(self, **kw) -> None: ...

class BarcodeCode11(_BarcodeWidget):
    codeName: Final = "Code11"
    def __init__(self, **kw) -> None: ...

class BarcodeFIM(_BarcodeWidget):
    codeName: Final = "FIM"
    def __init__(self, **kw) -> None: ...

class BarcodePOSTNET(_BarcodeWidget):
    codeName: Final = "POSTNET"
    def __init__(self, **kw) -> None: ...

class BarcodeUSPS_4State(_BarcodeWidget):
    codeName: Final = "USPS_4State"
    def __init__(self, **kw) -> None: ...

__all__ = (
    "BarcodeI2of5",
    "BarcodeCode128",
    "BarcodeStandard93",
    "BarcodeExtended93",
    "BarcodeStandard39",
    "BarcodeExtended39",
    "BarcodeMSI",
    "BarcodeCodabar",
    "BarcodeCode11",
    "BarcodeFIM",
    "BarcodePOSTNET",
    "BarcodeUSPS_4State",
)



import sys
from _typeshed import Incomplete
from typing import Any, Final, final
from typing_extensions import Self

version: str

def global_init(option: int) -> None: ...
def global_cleanup() -> None: ...
def version_info() -> (
    tuple[int, str, int, str, int, str, int, str, tuple[str, ...], Incomplete | None, int, Incomplete | None]
): ...

class error(Exception): ...

@final
class Curl:
    USERPWD: int
    def close(self) -> None: ...
    def setopt(self, option: int, value) -> None: ...
    def setopt_string(self, option: int, value: str) -> None: ...
    def perform(self) -> None: ...
    def perform_rb(self) -> bytes: ...
    def perform_rs(self) -> str: ...
    # For getinfo and getinfo_raw, the exact return type depends on the passed value:
    # http://pycurl.io/docs/latest/curlobject.html#pycurl.Curl.getinfo
    def getinfo(self, info: int) -> Any: ...
    def getinfo_raw(self, info: int) -> Any: ...
    def reset(self) -> None: ...
    def unsetopt(self, option: int): ...
    def pause(self, bitmask): ...
    def errstr(self) -> str: ...
    def duphandle(self) -> Self: ...
    def errstr_raw(self) -> bytes: ...
    if sys.platform == "linux" or sys.platform == "darwin":
        def set_ca_certs(self, value: bytes | str, /) -> None: ...

@final
class CurlMulti:
    def close(self) -> None: ...
    def add_handle(self, obj: Curl) -> None: ...
    def remove_handle(self, obj: Curl) -> None: ...
    def setopt(self, option: int, value) -> None: ...
    def perform(self) -> tuple[Incomplete, int]: ...
    def fdset(self) -> tuple[list[Incomplete], list[Incomplete], list[Incomplete]]: ...
    def select(self, timeout: float) -> int: ...
    def info_read(self, max_objects: int = ...) -> tuple[int, list[Incomplete], list[Incomplete]]: ...
    def socket_action(self, sockfd: int, ev_bitmask: int) -> tuple[int, int]: ...
    def assign(self, sockfd: int, socket, /): ...
    def socket_all(self) -> tuple[int, int]: ...
    def timeout(self) -> int: ...

@final
class CurlShare:
    def close(self) -> None: ...
    def setopt(self, option: int, value): ...

APPCONNECT_TIME_T: Final[int] = ...
CONNECT_TIME_T: Final[int] = ...
CONTENT_LENGTH_DOWNLOAD_T: Final[int] = ...
CONTENT_LENGTH_UPLOAD_T: Final[int] = ...
EARLYDATA_SENT_T: Final[int] = ...
FILETIME_T: Final[int] = ...
NAMELOOKUP_TIME_T: Final[int] = ...
POSTTRANSFER_TIME_T: Final[int] = ...
PRETRANSFER_TIME_T: Final[int] = ...
QUEUE_TIME_T: Final[int] = ...
REDIRECT_TIME_T: Final[int] = ...
SIZE_DOWNLOAD_T: Final[int] = ...
SIZE_UPLOAD_T: Final[int] = ...
SPEED_DOWNLOAD_T: Final[int] = ...
SPEED_UPLOAD_T: Final[int] = ...
STARTTRANSFER_TIME_T: Final[int] = ...
TOTAL_TIME_T: Final[int] = ...

ACCEPTTIMEOUT_MS: Final = 212
ACCEPT_ENCODING: Final = 10102
ADDRESS_SCOPE: Final = 171
APPCONNECT_TIME: Final = 3145761
APPEND: Final = 50
AUTOREFERER: Final = 58
AWS_SIGV4: Final = 10305
BUFFERSIZE: Final = 98
CAINFO: Final = 10065
CAINFO_BLOB: Final = 40309
CAPATH: Final = 10097
CLOSESOCKETFUNCTION: Final = 20208
COMPILE_LIBCURL_VERSION_NUM: Final = 525824
COMPILE_PY_VERSION_HEX: Final = 50925040
COMPILE_SSL_LIB: Final = "schannel"
CONDITION_UNMET: Final = 2097187
CONNECTTIMEOUT: Final = 78
CONNECTTIMEOUT_MS: Final = 156
CONNECT_ONLY: Final = 141
CONNECT_TIME: Final = 3145733
CONNECT_TO: Final = 10243
CONTENT_LENGTH_DOWNLOAD: Final = 3145743
CONTENT_LENGTH_UPLOAD: Final = 3145744
CONTENT_TYPE: Final = 1048594
COOKIE: Final = 10022
COOKIEFILE: Final = 10031
COOKIEJAR: Final = 10082
COOKIELIST: Final = 10135
COOKIESESSION: Final = 96
COPYPOSTFIELDS: Final = 10165
CRLF: Final = 27
CRLFILE: Final = 10169
CSELECT_ERR: Final = 4
CSELECT_IN: Final = 1
CSELECT_OUT: Final = 2
CURL_HTTP_VERSION_1_0: Final = 1
CURL_HTTP_VERSION_1_1: Final = 2
CURL_HTTP_VERSION_2: Final = 3
CURL_HTTP_VERSION_2TLS: Final = 4
CURL_HTTP_VERSION_2_0: Final = 3
CURL_HTTP_VERSION_2_PRIOR_KNOWLEDGE: Final = 5
CURL_HTTP_VERSION_3: Final = 30
CURL_HTTP_VERSION_3ONLY: Final = 31
CURL_HTTP_VERSION_LAST: Final = 32
CURL_HTTP_VERSION_NONE: Final = 0
CURL_VERSION_ALTSVC: Final = 16777216
CURL_VERSION_BROTLI: Final = 8388608
CURL_VERSION_GSASL: Final = 536870912
CURL_VERSION_HSTS: Final = 268435456
CURL_VERSION_HTTP3: Final = 33554432
CURL_VERSION_HTTPS_PROXY: Final = 2097152
CURL_VERSION_MULTI_SSL: Final = 4194304
CURL_VERSION_UNICODE: Final = 134217728
CURL_VERSION_ZSTD: Final = 67108864
CUSTOMREQUEST: Final = 10036
DEBUGFUNCTION: Final = 20094
DEFAULT_PROTOCOL: Final = 10238
DIRLISTONLY: Final = 48
DNS_CACHE_TIMEOUT: Final = 92
DNS_SERVERS: Final = 10211
DNS_USE_GLOBAL_CACHE: Final = 91
DOH_URL: Final = 10279
EFFECTIVE_URL: Final = 1048577
EFFECTIVE_METHOD: Final = 1048634
EGDSOCKET: Final = 10077
ENCODING: Final = 10102
EXPECT_100_TIMEOUT_MS: Final = 227
E_ABORTED_BY_CALLBACK: Final = 42
E_AGAIN: Final = 81
E_ALREADY_COMPLETE: Final = 99999
E_BAD_CALLING_ORDER: Final = 44
E_BAD_CONTENT_ENCODING: Final = 61
E_BAD_DOWNLOAD_RESUME: Final = 36
E_BAD_FUNCTION_ARGUMENT: Final = 43
E_BAD_PASSWORD_ENTERED: Final = 46
E_CALL_MULTI_PERFORM: Final = -1
E_CHUNK_FAILED: Final = 88
E_CONV_FAILED: Final = 75
E_CONV_REQD: Final = 76
E_COULDNT_CONNECT: Final = 7
E_COULDNT_RESOLVE_HOST: Final = 6
E_COULDNT_RESOLVE_PROXY: Final = 5
E_FAILED_INIT: Final = 2
E_FILESIZE_EXCEEDED: Final = 63
E_FILE_COULDNT_READ_FILE: Final = 37
E_FTP_ACCEPT_FAILED: Final = 10
E_FTP_ACCEPT_TIMEOUT: Final = 12
E_FTP_ACCESS_DENIED: Final = 9
E_FTP_BAD_DOWNLOAD_RESUME: Final = 36
E_FTP_BAD_FILE_LIST: Final = 87
E_FTP_CANT_GET_HOST: Final = 15
E_FTP_CANT_RECONNECT: Final = 16
E_FTP_COULDNT_GET_SIZE: Final = 32
E_FTP_COULDNT_RETR_FILE: Final = 19
E_FTP_COULDNT_SET_ASCII: Final = 29
E_FTP_COULDNT_SET_BINARY: Final = 17
E_FTP_COULDNT_SET_TYPE: Final = 17
E_FTP_COULDNT_STOR_FILE: Final = 25
E_FTP_COULDNT_USE_REST: Final = 31
E_FTP_PARTIAL_FILE: Final = 18
E_FTP_PORT_FAILED: Final = 30
E_FTP_PRET_FAILED: Final = 84
E_FTP_QUOTE_ERROR: Final = 21
E_FTP_SSL_FAILED: Final = 64
E_FTP_USER_PASSWORD_INCORRECT: Final = 10
E_FTP_WEIRD_227_FORMAT: Final = 14
E_FTP_WEIRD_PASS_REPLY: Final = 11
E_FTP_WEIRD_PASV_REPLY: Final = 13
E_FTP_WEIRD_SERVER_REPLY: Final = 8
E_FTP_WEIRD_USER_REPLY: Final = 12
E_FTP_WRITE_ERROR: Final = 20
E_FUNCTION_NOT_FOUND: Final = 41
E_GOT_NOTHING: Final = 52
E_HTTP2: Final = 16
E_HTTP_NOT_FOUND: Final = 22
E_HTTP_PORT_FAILED: Final = 45
E_HTTP_POST_ERROR: Final = 34
E_HTTP_RANGE_ERROR: Final = 33
E_HTTP_RETURNED_ERROR: Final = 22
E_INTERFACE_FAILED: Final = 45
E_LDAP_CANNOT_BIND: Final = 38
E_LDAP_INVALID_URL: Final = 62
E_LDAP_SEARCH_FAILED: Final = 39
E_LIBRARY_NOT_FOUND: Final = 40
E_LOGIN_DENIED: Final = 67
E_MALFORMAT_USER: Final = 24
E_MULTI_ADDED_ALREADY: Final = 7
E_MULTI_BAD_EASY_HANDLE: Final = 2
E_MULTI_BAD_HANDLE: Final = 1
E_MULTI_BAD_SOCKET: Final = 5
E_MULTI_CALL_MULTI_PERFORM: Final = -1
E_MULTI_CALL_MULTI_SOCKET: Final = -1
E_MULTI_INTERNAL_ERROR: Final = 4
E_MULTI_OK: Final = 0
E_MULTI_OUT_OF_MEMORY: Final = 3
E_MULTI_UNKNOWN_OPTION: Final = 6
E_NOT_BUILT_IN: Final = 4
E_NO_CONNECTION_AVAILABLE: Final = 89
E_OK: Final = 0
E_OPERATION_TIMEDOUT: Final = 28
E_OPERATION_TIMEOUTED: Final = 28
E_OUT_OF_MEMORY: Final = 27
E_PARTIAL_FILE: Final = 18
E_PEER_FAILED_VERIFICATION: Final = 60
E_QUOTE_ERROR: Final = 21
E_RANGE_ERROR: Final = 33
E_READ_ERROR: Final = 26
E_RECV_ERROR: Final = 56
E_REMOTE_ACCESS_DENIED: Final = 9
E_REMOTE_DISK_FULL: Final = 70
E_REMOTE_FILE_EXISTS: Final = 73
E_REMOTE_FILE_NOT_FOUND: Final = 78
E_RTSP_CSEQ_ERROR: Final = 85
E_RTSP_SESSION_ERROR: Final = 86
E_SEND_ERROR: Final = 55
E_SEND_FAIL_REWIND: Final = 65
E_SHARE_IN_USE: Final = 57
E_SSH: Final = 79
E_SSL_CACERT: Final = 60
E_SSL_CACERT_BADFILE: Final = 77
E_SSL_CERTPROBLEM: Final = 58
E_SSL_CIPHER: Final = 59
E_SSL_CONNECT_ERROR: Final = 35
E_SSL_CRL_BADFILE: Final = 82
E_SSL_ENGINE_INITFAILED: Final = 66
E_SSL_ENGINE_NOTFOUND: Final = 53
E_SSL_ENGINE_SETFAILED: Final = 54
E_SSL_INVALIDCERTSTATUS: Final = 91
E_SSL_ISSUER_ERROR: Final = 83
E_SSL_PEER_CERTIFICATE: Final = 60
E_SSL_PINNEDPUBKEYNOTMATCH: Final = 90
E_SSL_SHUTDOWN_FAILED: Final = 80
E_TELNET_OPTION_SYNTAX: Final = 49
E_TFTP_DISKFULL: Final = 70
E_TFTP_EXISTS: Final = 73
E_TFTP_ILLEGAL: Final = 71
E_TFTP_NOSUCHUSER: Final = 74
E_TFTP_NOTFOUND: Final = 68
E_TFTP_PERM: Final = 69
E_TFTP_UNKNOWNID: Final = 72
E_TOO_MANY_REDIRECTS: Final = 47
E_UNKNOWN_OPTION: Final = 48
E_UNKNOWN_TELNET_OPTION: Final = 48
E_UNSUPPORTED_PROTOCOL: Final = 1
E_UPLOAD_FAILED: Final = 25
E_URL_MALFORMAT: Final = 3
E_URL_MALFORMAT_USER: Final = 4
E_USE_SSL_FAILED: Final = 64
E_WRITE_ERROR: Final = 23
FAILONERROR: Final = 45
FILE: Final = 10001
FOLLOWLOCATION: Final = 52
FORBID_REUSE: Final = 75
FORM_BUFFER: Final = 11
FORM_BUFFERPTR: Final = 12
FORM_CONTENTS: Final = 4
FORM_CONTENTTYPE: Final = 14
FORM_FILE: Final = 10
FORM_FILENAME: Final = 16
FRESH_CONNECT: Final = 74
FTPAPPEND: Final = 50
FTPAUTH_DEFAULT: Final = 0
FTPAUTH_SSL: Final = 1
FTPAUTH_TLS: Final = 2
FTPLISTONLY: Final = 48
FTPMETHOD_DEFAULT: Final = 0
FTPMETHOD_MULTICWD: Final = 1
FTPMETHOD_NOCWD: Final = 2
FTPMETHOD_SINGLECWD: Final = 3
FTPPORT: Final = 10017
FTPSSLAUTH: Final = 129
FTPSSL_ALL: Final = 3
FTPSSL_CONTROL: Final = 2
FTPSSL_NONE: Final = 0
FTPSSL_TRY: Final = 1
FTP_ACCOUNT: Final = 10134
FTP_ALTERNATIVE_TO_USER: Final = 10147
FTP_CREATE_MISSING_DIRS: Final = 110
FTP_ENTRY_PATH: Final = 1048606
FTP_FILEMETHOD: Final = 138
FTP_RESPONSE_TIMEOUT: Final = 112
FTP_SKIP_PASV_IP: Final = 137
FTP_SSL: Final = 119
FTP_SSL_CCC: Final = 154
FTP_USE_EPRT: Final = 106
FTP_USE_EPSV: Final = 85
FTP_USE_PRET: Final = 188
GLOBAL_ACK_EINTR: Final = 4
GLOBAL_ALL: Final = 3
GLOBAL_DEFAULT: Final = 3
GLOBAL_NOTHING: Final = 0
GLOBAL_SSL: Final = 1
GLOBAL_WIN32: Final = 2
GSSAPI_DELEGATION: Final = 210
GSSAPI_DELEGATION_FLAG: Final = 2
GSSAPI_DELEGATION_NONE: Final = 0
GSSAPI_DELEGATION_POLICY_FLAG: Final = 1
HAPROXYPROTOCOL: Final = 274
HAPROXY_CLIENT_IP: Final = 10323
ECH: Final = 10325
HEADER: Final = 42
HEADERFUNCTION: Final = 20079
HEADEROPT: Final = 229
HEADER_SEPARATE: Final = 1
HEADER_SIZE: Final = 2097163
HEADER_UNIFIED: Final = 0
HTTP09_ALLOWED: Final = 285
HTTP200ALIASES: Final = 10104
HTTPAUTH: Final = 107
HTTPAUTH_ANY: Final = -17
HTTPAUTH_ANYSAFE: Final = -18
HTTPAUTH_AVAIL: Final = 2097175
HTTPAUTH_BASIC: Final = 1
HTTPAUTH_DIGEST: Final = 2
HTTPAUTH_DIGEST_IE: Final = 16
HTTPAUTH_GSSNEGOTIATE: Final = 4
HTTPAUTH_NEGOTIATE: Final = 4
HTTPAUTH_NONE: Final = 0
HTTPAUTH_NTLM: Final = 8
HTTPAUTH_NTLM_WB: Final = 32
HTTPAUTH_ONLY: Final = -2147483648
HTTPGET: Final = 80
HTTPHEADER: Final = 10023
HTTPPOST: Final = 10024
HTTPPROXYTUNNEL: Final = 61
HTTP_CODE: Final = 2097154
HTTP_CONNECTCODE: Final = 2097174
HTTP_CONTENT_DECODING: Final = 158
HTTP_TRANSFER_DECODING: Final = 157
HTTP_VERSION: Final = 84
IGNORE_CONTENT_LENGTH: Final = 136
INFILE: Final = 10009
INFILESIZE: Final = 30115
INFILESIZE_LARGE: Final = 30115
INFOTYPE_DATA_IN: Final = 3
INFOTYPE_DATA_OUT: Final = 4
INFOTYPE_HEADER_IN: Final = 1
INFOTYPE_HEADER_OUT: Final = 2
INFOTYPE_SSL_DATA_IN: Final = 5
INFOTYPE_SSL_DATA_OUT: Final = 6
INFOTYPE_TEXT: Final = 0
INFO_CERTINFO: Final = 4194338
INFO_COOKIELIST: Final = 4194332
INFO_FILETIME: Final = 2097166
INFO_HTTP_VERSION: Final = 2097198
INFO_RTSP_CLIENT_CSEQ: Final = 2097189
INFO_RTSP_CSEQ_RECV: Final = 2097191
INFO_RTSP_SERVER_CSEQ: Final = 2097190
INFO_RTSP_SESSION_ID: Final = 1048612
INTERFACE: Final = 10062
IOCMD_NOP: Final = 0
IOCMD_RESTARTREAD: Final = 1
IOCTLFUNCTION: Final = 20130
IOE_FAILRESTART: Final = 2
IOE_OK: Final = 0
IOE_UNKNOWNCMD: Final = 1
IPRESOLVE: Final = 113
IPRESOLVE_V4: Final = 1
IPRESOLVE_V6: Final = 2
IPRESOLVE_WHATEVER: Final = 0
ISSUERCERT: Final = 10170
ISSUERCERT_BLOB: Final = 40295
KEYPASSWD: Final = 10026
KHMATCH_MISMATCH: Final = 1
KHMATCH_MISSING: Final = 2
KHMATCH_OK: Final = 0
KHSTAT_DEFER: Final = 3
KHSTAT_FINE: Final = 1
KHSTAT_FINE_ADD_TO_FILE: Final = 0
KHSTAT_REJECT: Final = 2
KHTYPE_DSS: Final = 3
KHTYPE_RSA: Final = 2
KHTYPE_RSA1: Final = 1
KHTYPE_UNKNOWN: Final = 0
KRB4LEVEL: Final = 10063
KRBLEVEL: Final = 10063
LASTSOCKET: Final = 2097181
LOCALPORT: Final = 139
LOCALPORTRANGE: Final = 140
LOCAL_IP: Final = 1048617
LOCAL_PORT: Final = 2097194
LOCK_DATA_CONNECT: Final = 5
LOCK_DATA_COOKIE: Final = 2
LOCK_DATA_DNS: Final = 3
LOCK_DATA_PSL: Final = 6
LOCK_DATA_SSL_SESSION: Final = 4
LOGIN_OPTIONS: Final = 10224
LOW_SPEED_LIMIT: Final = 19
LOW_SPEED_TIME: Final = 20
MAIL_AUTH: Final = 10217
MAIL_FROM: Final = 10186
MAIL_RCPT: Final = 10187
MAXAGE_CONN: Final = 288
MAXCONNECTS: Final = 71
MAXFILESIZE: Final = 30117
MAXFILESIZE_LARGE: Final = 30117
MAXLIFETIME_CONN: Final = 314
PREREQFUNCTION: Final = 20312
PREREQFUNC_OK: Final = 0
PREREQFUNC_ABORT: Final = 1
MAXREDIRS: Final = 68
MAX_RECV_SPEED_LARGE: Final = 30146
MAX_SEND_SPEED_LARGE: Final = 30145
M_CHUNK_LENGTH_PENALTY_SIZE: Final = 30010
M_CONTENT_LENGTH_PENALTY_SIZE: Final = 30009
M_MAXCONNECTS: Final = 6
M_MAX_CONCURRENT_STREAMS: Final = 16
M_MAX_HOST_CONNECTIONS: Final = 7
M_MAX_PIPELINE_LENGTH: Final = 8
M_MAX_TOTAL_CONNECTIONS: Final = 13
M_PIPELINING: Final = 3
M_PIPELINING_SERVER_BL: Final = 10012
M_PIPELINING_SITE_BL: Final = 10011
M_SOCKETFUNCTION: Final = 20001
M_TIMERFUNCTION: Final = 20004
NAMELOOKUP_TIME: Final = 3145732
NETRC: Final = 51
NETRC_FILE: Final = 10118
NETRC_IGNORED: Final = 0
NETRC_OPTIONAL: Final = 1
NETRC_REQUIRED: Final = 2
NEW_DIRECTORY_PERMS: Final = 160
NEW_FILE_PERMS: Final = 159
NOBODY: Final = 44
NOPROGRESS: Final = 43
NOPROXY: Final = 10177
NOSIGNAL: Final = 99
NUM_CONNECTS: Final = 2097178
OPENSOCKETFUNCTION: Final = 20163
OPT_CERTINFO: Final = 172
OPT_COOKIELIST: Final = 10135
OPT_FILETIME: Final = 69
OPT_RTSP_CLIENT_CSEQ: Final = 193
OPT_RTSP_REQUEST: Final = 189
OPT_RTSP_SERVER_CSEQ: Final = 194
OPT_RTSP_SESSION_ID: Final = 10190
OPT_RTSP_STREAM_URI: Final = 10191
OPT_RTSP_TRANSPORT: Final = 10192
OS_ERRNO: Final = 2097177
PASSWORD: Final = 10174
PATH_AS_IS: Final = 234
PAUSE_ALL: Final = 5
PAUSE_CONT: Final = 0
PAUSE_RECV: Final = 1
PAUSE_SEND: Final = 4
PINNEDPUBLICKEY: Final = 10230
PIPEWAIT: Final = 237
PIPE_HTTP1: Final = 1
PIPE_MULTIPLEX: Final = 2
PIPE_NOTHING: Final = 0
POLL_IN: Final = 1
POLL_INOUT: Final = 3
POLL_NONE: Final = 0
POLL_OUT: Final = 2
POLL_REMOVE: Final = 4
PORT: Final = 3
POST: Final = 47
POST301: Final = 161
POSTFIELDS: Final = 10015
POSTFIELDSIZE: Final = 30120
POSTFIELDSIZE_LARGE: Final = 30120
POSTQUOTE: Final = 10039
POSTREDIR: Final = 161
PREQUOTE: Final = 10093
PRETRANSFER_TIME: Final = 3145734
PRE_PROXY: Final = 10262
PRIMARY_IP: Final = 1048608
PRIMARY_PORT: Final = 2097192
PROGRESSFUNCTION: Final = 20056
PROTOCOLS: Final = 181
PROTO_ALL: Final = -1
PROTO_DICT: Final = 512
PROTO_FILE: Final = 1024
PROTO_FTP: Final = 4
PROTO_FTPS: Final = 8
PROTO_GOPHER: Final = 33554432
PROTO_HTTP: Final = 1
PROTO_HTTPS: Final = 2
PROTO_IMAP: Final = 4096
PROTO_IMAPS: Final = 8192
PROTO_LDAP: Final = 128
PROTO_LDAPS: Final = 256
PROTO_POP3: Final = 16384
PROTO_POP3S: Final = 32768
PROTO_RTMP: Final = 524288
PROTO_RTMPE: Final = 2097152
PROTO_RTMPS: Final = 8388608
PROTO_RTMPT: Final = 1048576
PROTO_RTMPTE: Final = 4194304
PROTO_RTMPTS: Final = 16777216
PROTO_RTSP: Final = 262144
PROTO_SCP: Final = 16
PROTO_SFTP: Final = 32
PROTO_SMB: Final = 67108864
PROTO_SMBS: Final = 134217728
PROTO_SMTP: Final = 65536
PROTO_SMTPS: Final = 131072
PROTO_TELNET: Final = 64
PROTO_TFTP: Final = 2048
PROXY: Final = 10004
PROXYAUTH: Final = 111
PROXYAUTH_AVAIL: Final = 2097176
PROXYHEADER: Final = 10228
PROXYPASSWORD: Final = 10176
PROXYPORT: Final = 59
PROXYTYPE: Final = 101
PROXYTYPE_HTTP: Final = 0
PROXYTYPE_HTTP_1_0: Final = 1
PROXYTYPE_SOCKS4: Final = 4
PROXYTYPE_SOCKS4A: Final = 6
PROXYTYPE_SOCKS5: Final = 5
PROXYTYPE_SOCKS5_HOSTNAME: Final = 7
PROXYUSERNAME: Final = 10175
PROXYUSERPWD: Final = 10006
PROXY_CAINFO: Final = 10246
PROXY_CAINFO_BLOB: Final = 40310
PROXY_CAPATH: Final = 10247
PROXY_CRLFILE: Final = 10260
PROXY_ISSUERCERT: Final = 10296
PROXY_ISSUERCERT_BLOB: Final = 40297
PROXY_KEYPASSWD: Final = 10258
PROXY_PINNEDPUBLICKEY: Final = 10263
PROXY_SERVICE_NAME: Final = 10235
PROXY_SSLCERT: Final = 10254
PROXY_SSLCERTTYPE: Final = 10255
PROXY_SSLCERT_BLOB: Final = 40293
PROXY_SSLKEY: Final = 10256
PROXY_SSLKEYTYPE: Final = 10257
PROXY_SSLKEY_BLOB: Final = 40294
PROXY_SSLVERSION: Final = 250
PROXY_SSL_CIPHER_LIST: Final = 10259
PROXY_SSL_OPTIONS: Final = 261
PROXY_SSL_VERIFYHOST: Final = 249
PROXY_SSL_VERIFYPEER: Final = 248
PROXY_TLS13_CIPHERS: Final = 10277
PROXY_TLSAUTH_PASSWORD: Final = 10252
PROXY_TLSAUTH_TYPE: Final = 10253
PROXY_TLSAUTH_USERNAME: Final = 10251
PROXY_TRANSFER_MODE: Final = 166
PUT: Final = 54
QUOTE: Final = 10028
RANDOM_FILE: Final = 10076
RANGE: Final = 10007
READDATA: Final = 10009
READFUNCTION: Final = 20012
READFUNC_ABORT: Final = 268435456
READFUNC_PAUSE: Final = 268435457
REDIRECT_COUNT: Final = 2097172
REDIRECT_TIME: Final = 3145747
REDIRECT_URL: Final = 1048607
REDIR_POST_301: Final = 1
REDIR_POST_302: Final = 2
REDIR_POST_303: Final = 4
REDIR_POST_ALL: Final = 7
REDIR_PROTOCOLS: Final = 182
REFERER: Final = 10016
REQUEST_SIZE: Final = 2097164
REQUEST_TARGET: Final = 10266
RESOLVE: Final = 10203
RESPONSE_CODE: Final = 2097154
RESUME_FROM: Final = 30116
RESUME_FROM_LARGE: Final = 30116
RTSPREQ_ANNOUNCE: Final = 3
RTSPREQ_DESCRIBE: Final = 2
RTSPREQ_GET_PARAMETER: Final = 8
RTSPREQ_LAST: Final = 12
RTSPREQ_NONE: Final = 0
RTSPREQ_OPTIONS: Final = 1
RTSPREQ_PAUSE: Final = 6
RTSPREQ_PLAY: Final = 5
RTSPREQ_RECEIVE: Final = 11
RTSPREQ_RECORD: Final = 10
RTSPREQ_SETUP: Final = 4
RTSPREQ_SET_PARAMETER: Final = 9
RTSPREQ_TEARDOWN: Final = 7
SASL_IR: Final = 218
SEEKFUNCTION: Final = 20167
SEEKFUNC_CANTSEEK: Final = 2
SEEKFUNC_FAIL: Final = 1
SEEKFUNC_OK: Final = 0
SERVICE_NAME: Final = 10236
SHARE: Final = 10100
SH_SHARE: Final = 1
SH_UNSHARE: Final = 2
SIZE_DOWNLOAD: Final = 3145736
SIZE_UPLOAD: Final = 3145735
SOCKET_BAD: Final = -1
SOCKET_TIMEOUT: Final = -1
SOCKOPTFUNCTION: Final = 20148
SOCKOPT_ALREADY_CONNECTED: Final = 2
SOCKOPT_ERROR: Final = 1
SOCKOPT_OK: Final = 0
SOCKS5_GSSAPI_NEC: Final = 180
SOCKS5_GSSAPI_SERVICE: Final = 10179
SOCKTYPE_ACCEPT: Final = 1
SOCKTYPE_IPCXN: Final = 0
SPEED_DOWNLOAD: Final = 3145737
SPEED_UPLOAD: Final = 3145738
SSH_AUTH_AGENT: Final = 16
SSH_AUTH_ANY: Final = -1
SSH_AUTH_DEFAULT: Final = -1
SSH_AUTH_HOST: Final = 4
SSH_AUTH_KEYBOARD: Final = 8
SSH_AUTH_NONE: Final = 0
SSH_AUTH_PASSWORD: Final = 2
SSH_AUTH_PUBLICKEY: Final = 1
SSH_AUTH_TYPES: Final = 151
SSH_HOST_PUBLIC_KEY_MD5: Final = 10162
SSH_KEYFUNCTION: Final = 20184
SSH_KNOWNHOSTS: Final = 10183
SSH_PRIVATE_KEYFILE: Final = 10153
SSH_PUBLIC_KEYFILE: Final = 10152
SSLCERT: Final = 10025
SSLCERTPASSWD: Final = 10026
SSLCERTTYPE: Final = 10086
SSLCERT_BLOB: Final = 40291
SSLENGINE: Final = 10089
SSLENGINE_DEFAULT: Final = 90
SSLKEY: Final = 10087
SSLKEYPASSWD: Final = 10026
SSLKEYTYPE: Final = 10088
SSLKEY_BLOB: Final = 40292
SSLOPT_ALLOW_BEAST: Final = 1
SSLOPT_NO_REVOKE: Final = 2
SSLVERSION: Final = 32
SSLVERSION_DEFAULT: Final = 0
SSLVERSION_MAX_DEFAULT: Final = 65536
SSLVERSION_MAX_TLSv1_0: Final = 262144
SSLVERSION_MAX_TLSv1_1: Final = 327680
SSLVERSION_MAX_TLSv1_2: Final = 393216
SSLVERSION_MAX_TLSv1_3: Final = 458752
SSLVERSION_SSLv2: Final = 2
SSLVERSION_SSLv3: Final = 3
SSLVERSION_TLSv1: Final = 0
SSLVERSION_TLSv1_0: Final = 4
SSLVERSION_TLSv1_1: Final = 5
SSLVERSION_TLSv1_2: Final = 6
SSLVERSION_TLSv1_3: Final = 7
SSL_CIPHER_LIST: Final = 10083
SSL_ENABLE_ALPN: Final = 226
SSL_ENABLE_NPN: Final = 225
SSL_ENGINES: Final = 4194331
SSL_FALSESTART: Final = 233
SSL_OPTIONS: Final = 216
SSL_SESSIONID_CACHE: Final = 150
SSL_VERIFYHOST: Final = 81
SSL_VERIFYPEER: Final = 64
SSL_VERIFYRESULT: Final = 2097165
SSL_VERIFYSTATUS: Final = 232
STARTTRANSFER_TIME: Final = 3145745
STDERR: Final = 10037
TCP_FASTOPEN: Final = 244
TCP_KEEPALIVE: Final = 213
TCP_KEEPIDLE: Final = 214
TCP_KEEPINTVL: Final = 215
TCP_NODELAY: Final = 121
TELNETOPTIONS: Final = 10070
TFTP_BLKSIZE: Final = 178
TIMECONDITION: Final = 33
TIMECONDITION_IFMODSINCE: Final = 1
TIMECONDITION_IFUNMODSINCE: Final = 2
TIMECONDITION_LASTMOD: Final = 3
TIMECONDITION_NONE: Final = 0
TIMEOUT: Final = 13
TIMEOUT_MS: Final = 155
TIMEVALUE: Final = 34
TLS13_CIPHERS: Final = 10276
TLSAUTH_PASSWORD: Final = 10205
TLSAUTH_TYPE: Final = 10206
TLSAUTH_USERNAME: Final = 10204
TOTAL_TIME: Final = 3145731
TRANSFERTEXT: Final = 53
TRANSFER_ENCODING: Final = 207
UNIX_SOCKET_PATH: Final = 10231
UNRESTRICTED_AUTH: Final = 105
UPLOAD: Final = 46
UPLOAD_BUFFERSIZE: Final = 280
URL: Final = 10002
USERAGENT: Final = 10018
USERNAME: Final = 10173
USERPWD: Final = 10005
USESSL_ALL: Final = 3
USESSL_CONTROL: Final = 2
USESSL_NONE: Final = 0
USESSL_TRY: Final = 1
USE_SSL: Final = 119
VERBOSE: Final = 41
VERSION_ALTSVC: Final = 16777216
VERSION_ASYNCHDNS: Final = 128
VERSION_BROTLI: Final = 8388608
VERSION_CONV: Final = 4096
VERSION_CURLDEBUG: Final = 8192
VERSION_DEBUG: Final = 64
VERSION_GSASL: Final = 536870912
VERSION_GSSAPI: Final = 131072
VERSION_GSSNEGOTIATE: Final = 32
VERSION_HSTS: Final = 268435456
VERSION_HTTP2: Final = 65536
VERSION_HTTP3: Final = 33554432
VERSION_HTTPS_PROXY: Final = 2097152
VERSION_IDN: Final = 1024
VERSION_IPV6: Final = 1
VERSION_KERBEROS4: Final = 2
VERSION_KERBEROS5: Final = 262144
VERSION_LARGEFILE: Final = 512
VERSION_LIBZ: Final = 8
VERSION_MULTI_SSL: Final = 4194304
VERSION_NTLM: Final = 16
VERSION_NTLM_WB: Final = 32768
VERSION_PSL: Final = 1048576
VERSION_SPNEGO: Final = 256
VERSION_SSL: Final = 4
VERSION_SSPI: Final = 2048
VERSION_TLSAUTH_SRP: Final = 16384
VERSION_UNICODE: Final = 134217728
VERSION_UNIX_SOCKETS: Final = 524288
VERSION_ZSTD: Final = 67108864
WILDCARDMATCH: Final = 197
WRITEDATA: Final = 10001
WRITEFUNCTION: Final = 20011
WRITEFUNC_PAUSE: Final = 268435457
WRITEHEADER: Final = 10029
XFERINFOFUNCTION: Final = 20219
XOAUTH2_BEARER: Final = 10220



from _typeshed import SupportsRead
from typing import Final
from xml.dom.minidom import Document
from xml.sax.xmlreader import XMLReader

__origin__: Final = "xml.dom.minidom"

def parse(
    file: str | SupportsRead[bytes | str],
    parser: XMLReader | None = None,
    bufsize: int | None = None,
    forbid_dtd: bool = False,
    forbid_entities: bool = True,
    forbid_external: bool = True,
) -> Document: ...
def parseString(
    string: str,
    parser: XMLReader | None = None,
    forbid_dtd: bool = False,
    forbid_entities: bool = True,
    forbid_external: bool = True,
) -> Document: ...



__all__ = ["check", "checkPath", "checkRecursive", "iterSourceCode", "main"]
from pyflakes.api import (
    check as check,
    checkPath as checkPath,
    checkRecursive as checkRecursive,
    iterSourceCode as iterSourceCode,
    main as main,
)



from .base import AuthenticationBase

class Delegated(AuthenticationBase):
    def get_token(
        self,
        target: str,
        api_type: str,
        grant_type: str,
        id_token: str | None = None,
        refresh_token: str | None = None,
        scope: str = "openid",
    ): ...



from typing import Final, TypeVar

_T = TypeVar("_T")

__author__: Final[str]
__copyright__: Final[str]
__contributors__: Final[list[str]]
__version__: Final[str]
__license__: Final[str]

escape_range: list[tuple[int, int]]

def encode(c: str) -> str: ...
def iri2uri(uri: _T) -> _T: ...



from _typeshed import Incomplete
from typing import Final

from authlib.jose import JWTClaims

class ClientRegistrationEndpoint:
    ENDPOINT_NAME: Final = "client_registration"
    software_statement_alg_values_supported: Incomplete
    server: Incomplete
    claims_classes: list[type[Incomplete]]
    def __init__(self, server=None, claims_classes: list[type[Incomplete]] | None = None) -> None: ...
    def __call__(self, request) -> dict[Incomplete, Incomplete]: ...
    def create_registration_response(self, request): ...
    def extract_client_metadata(self, request): ...
    def extract_software_statement(self, software_statement, request) -> JWTClaims: ...
    def generate_client_info(self, request) -> dict[str, Incomplete]: ...
    def generate_client_registration_info(self, client, request) -> None: ...
    def create_endpoint_request(self, request): ...
    def generate_client_id(self, request) -> str: ...
    def generate_client_secret(self, request) -> str: ...
    def get_server_metadata(self) -> None: ...
    def authenticate_token(self, request) -> None: ...
    def resolve_public_key(self, request) -> None: ...
    def save_client(self, client_info, client_metadata, request) -> None: ...



__all__ = ["NamedType", "OptionalNamedType", "DefaultedNamedType", "NamedTypes"]

class NamedType:
    isOptional: bool
    isDefaulted: bool
    def __init__(self, name, asn1Object, openType: type | None = None) -> None: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...
    def __hash__(self): ...
    def __getitem__(self, idx): ...
    def __iter__(self): ...
    @property
    def name(self): ...
    @property
    def asn1Object(self): ...
    @property
    def openType(self): ...
    def getName(self): ...
    def getType(self): ...

class OptionalNamedType(NamedType):
    isOptional: bool

class DefaultedNamedType(NamedType):
    isDefaulted: bool

class NamedTypes:
    def __init__(self, *namedTypes, **kwargs) -> None: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __lt__(self, other): ...
    def __le__(self, other): ...
    def __gt__(self, other): ...
    def __ge__(self, other): ...
    def __hash__(self): ...
    def __getitem__(self, idx): ...
    def __contains__(self, key) -> bool: ...
    def __iter__(self): ...
    def __bool__(self) -> bool: ...
    def __len__(self) -> int: ...
    def values(self): ...
    def keys(self): ...
    def items(self): ...
    def clone(self): ...

    class PostponedError:
        def __init__(self, errorMsg) -> None: ...
        def __getitem__(self, item) -> None: ...

    def getTypeByPosition(self, idx): ...
    def getPositionByType(self, tagSet): ...
    def getNameByPosition(self, idx): ...
    def getPositionByName(self, name): ...
    def getTagMapNearPosition(self, idx): ...
    def getPositionNearType(self, tagSet, idx): ...
    @property
    def minTagSet(self): ...
    @property
    def tagMap(self): ...
    @property
    def tagMapUnique(self): ...
    @property
    def hasOptionalOrDefault(self): ...
    @property
    def hasOpenTypes(self): ...
    @property
    def namedTypes(self): ...
    @property
    def requiredComponents(self): ...



from __future__ import annotations

import io
import pathlib
import zipfile
from typing import Literal

###
# Tests for `zipfile.ZipFile`
###

p = pathlib.Path("test.zip")


class CustomPathObj:
    def __init__(self, path: str) -> None:
        self.path = path

    def __fspath__(self) -> str:
        return self.path


class NonPathObj:
    def __init__(self, path: str) -> None:
        self.path = path


class ReadableObj:
    def seek(self, offset: int, whence: int = 0) -> int:
        return 0

    def read(self, n: int | None = -1) -> bytes:
        return b"test"


class TellableObj:
    def tell(self) -> int:
        return 0


class WriteableObj:
    def close(self) -> None:
        pass

    def write(self, b: bytes) -> int:
        return len(b)

    def flush(self) -> None:
        pass


class ReadTellableObj(ReadableObj):
    def tell(self) -> int:
        return 0


class SeekTellObj:
    def seek(self, offset: int, whence: int = 0) -> int:
        return 0

    def tell(self) -> int:
        return 0


def write_zip(mode: Literal["r", "w", "x", "a"]) -> None:
    # Test any mode with `pathlib.Path`
    with zipfile.ZipFile(p, mode) as z:
        z.writestr("test.txt", "test")

    # Test any mode with `str` path
    with zipfile.ZipFile("test.zip", mode) as z:
        z.writestr("test.txt", "test")

    # Test any mode with `os.PathLike` object
    with zipfile.ZipFile(CustomPathObj("test.zip"), mode) as z:
        z.writestr("test.txt", "test")

    # Non-PathLike object should raise an error
    with zipfile.ZipFile(NonPathObj("test.zip"), mode) as z:  # type: ignore
        z.writestr("test.txt", "test")

    # IO[bytes] like-obj should work for any mode.
    io_obj = io.BytesIO(b"This is a test")
    with zipfile.ZipFile(io_obj, mode) as z:
        z.writestr("test.txt", "test")

    # Readable object should not work for any mode.
    with zipfile.ZipFile(ReadableObj(), mode) as z:  # type: ignore
        z.writestr("test.txt", "test")

    # Readable object should work for "r" mode.
    with zipfile.ZipFile(ReadableObj(), "r") as z:
        z.writestr("test.txt", "test")

    # Readable/tellable object should work for "a" mode.
    with zipfile.ZipFile(ReadTellableObj(), "a") as z:
        z.writestr("test.txt", "test")

    # If it doesn't have 'tell' method, it should raise an error.
    with zipfile.ZipFile(ReadableObj(), "a") as z:  # type: ignore
        z.writestr("test.txt", "test")

    # Readable object should not work for "w" mode.
    with zipfile.ZipFile(ReadableObj(), "w") as z:  # type: ignore
        z.writestr("test.txt", "test")

    # Tellable object should not work for any mode.
    with zipfile.ZipFile(TellableObj(), mode) as z:  # type: ignore
        z.writestr("test.txt", "test")

    # Tellable object shouldn't work for "w" mode.
    # As `__del__` will call close.
    with zipfile.ZipFile(TellableObj(), "w") as z:  # type: ignore
        z.writestr("test.txt", "test")

    # Writeable object should not work for any mode.
    with zipfile.ZipFile(WriteableObj(), mode) as z:  # type: ignore
        z.writestr("test.txt", "test")

    # Writeable object should work for "w" mode.
    with zipfile.ZipFile(WriteableObj(), "w") as z:
        z.writestr("test.txt", "test")

    # Seekable and Tellable object should not work for any mode.
    with zipfile.ZipFile(SeekTellObj(), mode) as z:  # type: ignore
        z.writestr("test.txt", "test")

    # Seekable and Tellable object shouldn't work for "w" mode.
    # Cause `__del__` will call close.
    with zipfile.ZipFile(SeekTellObj(), "w") as z:  # type: ignore
        z.writestr("test.txt", "test")



from _typeshed import ConvertibleToInt
from collections.abc import Sequence
from typing import Any, Generic, NamedTuple, Protocol, TypeVar, overload, type_check_only
from typing_extensions import TypeAlias

from ._types import ErrorCorrect, MaskPattern
from .image.base import BaseImage
from .image.pil import PilImage
from .image.pure import PyPNGImage
from .util import QRData

ModulesType: TypeAlias = list[list[bool | None]]
precomputed_qr_blanks: dict[int, ModulesType]

_DefaultImage: TypeAlias = PilImage | PyPNGImage  # PilImage if Pillow is installed, PyPNGImage otherwise
_AnySeq = TypeVar("_AnySeq", bound=Sequence[Any])

@overload
def make(
    data: QRData | bytes | str,
    *,
    version: ConvertibleToInt | None = None,
    error_correction: ErrorCorrect = 0,
    box_size: ConvertibleToInt = 10,
    border: ConvertibleToInt = 4,
    image_factory: None = None,
    mask_pattern: MaskPattern | None = None,
) -> _DefaultImage: ...
@overload
def make(
    data: QRData | bytes | str,
    *,
    version: ConvertibleToInt | None = None,
    error_correction: ErrorCorrect = 0,
    box_size: ConvertibleToInt = 10,
    border: ConvertibleToInt = 4,
    image_factory: type[GenericImage],
    mask_pattern: MaskPattern | None = None,
) -> GenericImage: ...
def copy_2d_array(x: Sequence[_AnySeq]) -> list[_AnySeq]: ...

class ActiveWithNeighbors(NamedTuple):
    NW: bool
    N: bool
    NE: bool
    W: bool
    me: bool
    E: bool
    SW: bool
    S: bool
    SE: bool
    def __bool__(self) -> bool: ...

GenericImage = TypeVar("GenericImage", bound=BaseImage)  # noqa: Y001
GenericImageLocal = TypeVar("GenericImageLocal", bound=BaseImage)  # noqa: Y001

@type_check_only
class _TTYWriter(Protocol):
    def isatty(self) -> bool: ...
    def write(self, s: str, /) -> object: ...
    def flush(self) -> object: ...

class QRCode(Generic[GenericImage]):
    modules: ModulesType
    error_correction: ErrorCorrect
    box_size: int
    border: int
    image_factory: type[GenericImage] | None
    @overload
    def __init__(
        self,
        version: ConvertibleToInt | None,
        error_correction: ErrorCorrect,
        box_size: ConvertibleToInt,
        border: ConvertibleToInt,
        image_factory: type[GenericImage],
        mask_pattern: MaskPattern | None = None,
    ) -> None: ...
    @overload
    def __init__(
        self,
        version: ConvertibleToInt | None = None,
        error_correction: ErrorCorrect = 0,
        box_size: ConvertibleToInt = 10,
        border: ConvertibleToInt = 4,
        *,
        image_factory: type[GenericImage],
        mask_pattern: MaskPattern | None = None,
    ) -> None: ...
    @overload
    def __init__(
        self: QRCode[_DefaultImage],
        version: ConvertibleToInt | None = None,
        error_correction: ErrorCorrect = 0,
        box_size: ConvertibleToInt = 10,
        border: ConvertibleToInt = 4,
        image_factory: None = None,
        mask_pattern: MaskPattern | None = None,
    ) -> None: ...
    @property
    def version(self) -> int: ...
    @version.setter
    def version(self, value: ConvertibleToInt | None) -> None: ...
    @property
    def mask_pattern(self) -> MaskPattern | None: ...
    @mask_pattern.setter
    def mask_pattern(self, pattern: MaskPattern | None) -> None: ...
    modules_count: int
    data_cache: list[int]
    data_list: list[QRData]
    def clear(self) -> None: ...
    def add_data(self, data: QRData | bytes | str, optimize: int = 20) -> None: ...
    def make(self, fit: bool = True) -> None: ...
    def makeImpl(self, test: bool, mask_pattern: MaskPattern) -> None: ...
    def setup_position_probe_pattern(self, row: int, col: int) -> None: ...
    def best_fit(self, start: int | None = None) -> int: ...
    def best_mask_pattern(self) -> int: ...
    def print_tty(self, out: _TTYWriter | None = None) -> None: ...
    def print_ascii(self, out: _TTYWriter | None = None, tty: bool = False, invert: bool = False) -> None: ...
    # kwargs are passed on to the specific image factory used, and in turn passed through to
    # their make_image method.
    @overload
    def make_image(self, image_factory: None = None, **kwargs: Any) -> GenericImage: ...
    @overload
    def make_image(self, image_factory: type[GenericImageLocal], **kwargs: Any) -> GenericImageLocal: ...
    def is_constrained(self, row: int, col: int) -> bool: ...
    def setup_timing_pattern(self) -> None: ...
    def setup_position_adjust_pattern(self) -> None: ...
    def setup_type_number(self, test: bool) -> None: ...
    def setup_type_info(self, test: bool, mask_pattern: MaskPattern) -> None: ...
    def map_data(self, data: Sequence[int], mask_pattern: MaskPattern) -> None: ...
    def get_matrix(self) -> list[list[bool]]: ...
    def active_with_neighbors(self, row: int, col: int) -> ActiveWithNeighbors: ...



from _typeshed import Incomplete

from networkx.classes import Graph
from networkx.utils.backends import _dispatchable

__all__ = [
    "balanced_tree",
    "barbell_graph",
    "binomial_tree",
    "complete_graph",
    "complete_multipartite_graph",
    "circular_ladder_graph",
    "circulant_graph",
    "cycle_graph",
    "dorogovtsev_goltsev_mendes_graph",
    "empty_graph",
    "full_rary_tree",
    "kneser_graph",
    "ladder_graph",
    "lollipop_graph",
    "null_graph",
    "path_graph",
    "star_graph",
    "tadpole_graph",
    "trivial_graph",
    "turan_graph",
    "wheel_graph",
]

@_dispatchable
def full_rary_tree(r, n, create_using=None): ...
@_dispatchable
def kneser_graph(n, k) -> Graph[Incomplete]: ...
@_dispatchable
def balanced_tree(r, h, create_using=None): ...
@_dispatchable
def barbell_graph(m1, m2, create_using=None): ...
@_dispatchable
def binomial_tree(n, create_using=None): ...
@_dispatchable
def complete_graph(n, create_using=None): ...
@_dispatchable
def circular_ladder_graph(n, create_using=None): ...
@_dispatchable
def circulant_graph(n, offsets, create_using=None): ...
@_dispatchable
def cycle_graph(n, create_using=None): ...
@_dispatchable
def dorogovtsev_goltsev_mendes_graph(n, create_using=None): ...
@_dispatchable
def empty_graph(n: Incomplete | int = 0, create_using=None, default=...): ...
@_dispatchable
def ladder_graph(n, create_using=None): ...
@_dispatchable
def lollipop_graph(m, n, create_using=None): ...
@_dispatchable
def null_graph(create_using=None): ...
@_dispatchable
def path_graph(n, create_using=None): ...
@_dispatchable
def star_graph(n, create_using=None): ...
@_dispatchable
def tadpole_graph(m, n, create_using=None) -> Graph[Incomplete] | Incomplete: ...
@_dispatchable
def trivial_graph(create_using=None): ...
@_dispatchable
def turan_graph(n, r): ...
@_dispatchable
def wheel_graph(n, create_using=None): ...
@_dispatchable
def complete_multipartite_graph(*subset_sizes): ...



from __future__ import annotations

import sys
from multiprocessing.connection import Pipe

if sys.platform != "win32":
    from multiprocessing.connection import Connection
else:
    from multiprocessing.connection import PipeConnection as Connection


# Unfortunately, we cannot validate that both connections have the same, but inverted generic types,
# since TypeVars scoped entirely within a return annotation is unspecified in the spec.
# Pipe[str, int]() -> tuple[Connection[str, int], Connection[int, str]]

a: Connection[str, int]
b: Connection[int, str]
a, b = Pipe()

connections: tuple[Connection[str, int], Connection[int, str]] = Pipe()
a, b = connections

a.send("test")
a.send(0)  # type: ignore
test1: str = b.recv()
test2: int = b.recv()  # type: ignore

b.send("test")  # type: ignore
b.send(0)
test3: str = a.recv()  # type: ignore
test4: int = a.recv()



from collections.abc import Sequence
from typing import Final

from docutils.parsers.rst import Directive

__docformat__: Final = "reStructuredText"

class Contents(Directive):
    backlinks_values: Sequence[str]
    def backlinks(arg): ...

class Sectnum(Directive): ...
class Header(Directive): ...
class Footer(Directive): ...



from _typeshed import Incomplete
from collections.abc import Callable, Iterator, Sequence
from re import Match
from typing import Any, Generic, TypeVar

from parsimonious.exceptions import VisitationError as VisitationError
from parsimonious.expressions import Expression
from parsimonious.grammar import Grammar

class Node:
    __slots__ = ["expr", "full_text", "start", "end", "children"]
    expr: Expression
    full_text: str
    start: int
    end: int
    children: Sequence[Node]
    def __init__(
        self, expr: Expression, full_text: str, start: int, end: int, children: Sequence[Node] | None = None
    ) -> None: ...
    @property
    def expr_name(self) -> str: ...
    def __iter__(self) -> Iterator[Node]: ...
    @property
    def text(self) -> str: ...
    def prettily(self, error: Node | None = None) -> str: ...
    def __repr__(self, top_level: bool = True) -> str: ...

class RegexNode(Node):
    __slots__ = ["match"]
    match: Match[str]

class RuleDecoratorMeta(type): ...

_VisitResultT = TypeVar("_VisitResultT")
_ChildT = TypeVar("_ChildT")

class NodeVisitor(Generic[_VisitResultT], metaclass=RuleDecoratorMeta):
    grammar: Grammar | Incomplete
    unwrapped_exceptions: tuple[type[BaseException], ...]
    def visit(self, node: Node) -> _VisitResultT: ...
    def generic_visit(self, node: Node, visited_children: Sequence[Any]): ...
    def parse(self, text: str, pos: int = 0) -> _VisitResultT: ...
    def match(self, text: str, pos: int = 0) -> _VisitResultT: ...
    def lift_child(self, node: Node, children: Sequence[_ChildT]) -> _ChildT: ...

_CallableT = TypeVar("_CallableT", bound=Callable[..., Any])

def rule(rule_string: str) -> Callable[[_CallableT], _CallableT]: ...



from _typeshed import Incomplete
from collections.abc import Generator
from typing import ClassVar

__all__ = ["generate_network_text", "write_network_text"]

class BaseGlyphs:
    @classmethod
    def as_dict(cls) -> dict[str, str]: ...

class AsciiBaseGlyphs(BaseGlyphs):
    empty: ClassVar[str]
    newtree_last: ClassVar[str]
    newtree_mid: ClassVar[str]
    endof_forest: ClassVar[str]
    within_forest: ClassVar[str]
    within_tree: ClassVar[str]

class AsciiDirectedGlyphs(AsciiBaseGlyphs):
    last: ClassVar[str]
    mid: ClassVar[str]
    backedge: ClassVar[str]
    vertical_edge: ClassVar[str]

class AsciiUndirectedGlyphs(AsciiBaseGlyphs):
    last: ClassVar[str]
    mid: ClassVar[str]
    backedge: ClassVar[str]
    vertical_edge: ClassVar[str]

class UtfBaseGlyphs(BaseGlyphs):
    empty: ClassVar[str]
    newtree_last: ClassVar[str]
    newtree_mid: ClassVar[str]
    endof_forest: ClassVar[str]
    within_forest: ClassVar[str]
    within_tree: ClassVar[str]

class UtfDirectedGlyphs(UtfBaseGlyphs):
    last: ClassVar[str]
    mid: ClassVar[str]
    backedge: ClassVar[str]
    vertical_edge: ClassVar[str]

class UtfUndirectedGlyphs(UtfBaseGlyphs):
    last: ClassVar[str]
    mid: ClassVar[str]
    backedge: ClassVar[str]
    vertical_edge: ClassVar[str]

def generate_network_text(
    graph, with_labels: bool = True, sources=None, max_depth=None, ascii_only: bool = False, vertical_chains: bool = False
) -> Generator[Incomplete, None, Incomplete]: ...
def write_network_text(
    graph,
    path=None,
    with_labels: bool = True,
    sources=None,
    max_depth=None,
    ascii_only: bool = False,
    end: str = "\n",
    vertical_chains=False,
) -> None: ...



from _typeshed import Incomplete

from authlib.common.errors import AuthlibBaseError

def extract_header(header_segment: bytes, error_cls: AuthlibBaseError) -> dict[Incomplete, Incomplete]: ...
def extract_segment(segment: bytes, error_cls: AuthlibBaseError, name: str = "payload") -> bytes: ...
def ensure_dict(s: object, structure_name: str) -> dict[Incomplete, Incomplete]: ...



import functools
from dataclasses import dataclass

from yt_dlp.utils._utils import NO_DEFAULT
from yt_dlp.YoutubeDL import YoutubeDL

__all__ = ["Updater"]

@dataclass
class UpdateInfo:
    tag: str
    version: str | None = ...
    requested_version: str | None = ...
    commit: str | None = ...
    binary_name: str | None = ...
    checksum: str | None = ...

class Updater:
    ydl: YoutubeDL
    requested_channel: str
    requested_tag: str | None
    requested_repo: str | None
    def __init__(self, ydl: YoutubeDL, target: str | None = None) -> None: ...
    @property
    def current_version(self) -> str: ...
    @property
    def current_commit(self) -> str: ...
    def query_update(self, *, _output: bool = False) -> UpdateInfo | None: ...
    def update(self, update_info: type[NO_DEFAULT] | None | UpdateInfo = ...) -> bool | None: ...
    @functools.cached_property
    def filename(self) -> str: ...
    @functools.cached_property
    def cmd(self) -> list[str]: ...
    def restart(self) -> int: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import sys
import typing

import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import tensorflow.core.framework.tensor_shape_pb2
import tensorflow.core.framework.types_pb2
import tensorflow.core.framework.variable_pb2
import tensorflow.core.framework.versions_pb2
import tensorflow.core.protobuf.struct_pb2
import tensorflow.core.protobuf.trackable_object_graph_pb2

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class SavedObjectGraph(google.protobuf.message.Message):
    """SavedObjectGraph shares some structure with TrackableObjectGraph, but
    SavedObjectGraph belongs to the MetaGraph and contains pointers to functions
    and type information, while TrackableObjectGraph lives in the checkpoint
    and contains pointers only to variable values.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class ConcreteFunctionsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___SavedConcreteFunction: ...
        def __init__(self, *, key: builtins.str | None = ..., value: global___SavedConcreteFunction | None = ...) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    NODES_FIELD_NUMBER: builtins.int
    CONCRETE_FUNCTIONS_FIELD_NUMBER: builtins.int
    @property
    def nodes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___SavedObject]:
        """Flattened list of objects in the object graph.

        The position of the object in this list indicates its id.
        Nodes[0] is considered the root node.
        """

    @property
    def concrete_functions(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___SavedConcreteFunction]:
        """Information about captures and output structures in concrete functions.
        Referenced from SavedBareConcreteFunction and SavedFunction.
        """

    def __init__(
        self,
        *,
        nodes: collections.abc.Iterable[global___SavedObject] | None = ...,
        concrete_functions: collections.abc.Mapping[builtins.str, global___SavedConcreteFunction] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["concrete_functions", b"concrete_functions", "nodes", b"nodes"]) -> None: ...

global___SavedObjectGraph = SavedObjectGraph

@typing.final
class SavedObject(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class SaveableObjectsEntry(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        KEY_FIELD_NUMBER: builtins.int
        VALUE_FIELD_NUMBER: builtins.int
        key: builtins.str
        @property
        def value(self) -> global___SaveableObject: ...
        def __init__(self, *, key: builtins.str | None = ..., value: global___SaveableObject | None = ...) -> None: ...
        def HasField(self, field_name: typing.Literal["value", b"value"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["key", b"key", "value", b"value"]) -> None: ...

    CHILDREN_FIELD_NUMBER: builtins.int
    DEPENDENCIES_FIELD_NUMBER: builtins.int
    SLOT_VARIABLES_FIELD_NUMBER: builtins.int
    USER_OBJECT_FIELD_NUMBER: builtins.int
    ASSET_FIELD_NUMBER: builtins.int
    FUNCTION_FIELD_NUMBER: builtins.int
    VARIABLE_FIELD_NUMBER: builtins.int
    BARE_CONCRETE_FUNCTION_FIELD_NUMBER: builtins.int
    CONSTANT_FIELD_NUMBER: builtins.int
    RESOURCE_FIELD_NUMBER: builtins.int
    CAPTURED_TENSOR_FIELD_NUMBER: builtins.int
    SAVEABLE_OBJECTS_FIELD_NUMBER: builtins.int
    REGISTERED_NAME_FIELD_NUMBER: builtins.int
    SERIALIZED_USER_PROTO_FIELD_NUMBER: builtins.int
    REGISTERED_SAVER_FIELD_NUMBER: builtins.int
    registered_name: builtins.str
    """The fields below are filled when the user serializes a registered Trackable
    class or an object with a registered saver function.

    Registered classes may save additional metadata and supersede the
    default loading process where nodes are recreated from the proto.
    If the registered class cannot be found, then the object will load as one
    one of the default trackable objects: Autotrackable (a class similar to
    tf.Module), tf.function, or tf.Variable.

    Unlike SaveableObjects, which store the functions for saving and restoring
    from tensors, registered savers allow Trackables to write checkpoint shards
    directly (e.g. for performance or coordination reasons).
    *All registered savers must be available when loading the SavedModel.*

    The name of the registered class of the form "{package}.{class_name}".
    This field is used to search for the registered class at loading time.
    """
    registered_saver: builtins.str
    """String name of the registered saver. At most one of `saveable_objects` or
    `registered_saver` is defined for each SavedObject.
    """
    @property
    def children(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        tensorflow.core.protobuf.trackable_object_graph_pb2.TrackableObjectGraph.TrackableObject.ObjectReference
    ]:
        """Objects which this object depends on: named edges in the dependency
        graph.

        Note: All kinds of SavedObject may have children, except
        "constant" and "captured_tensor".
        """

    @property
    def dependencies(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        tensorflow.core.protobuf.trackable_object_graph_pb2.TrackableObjectGraph.TrackableObject.ObjectReference
    ]:
        """Ordered list of dependencies that must be loaded before this object.
        SavedModel loads with the bottom-up approach, by first creating all objects
        (in the order defined by the dependencies), then connecting the edges.
        """

    @property
    def slot_variables(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[
        tensorflow.core.protobuf.trackable_object_graph_pb2.TrackableObjectGraph.TrackableObject.SlotVariableReference
    ]:
        """Slot variables owned by this object. This describes the three-way
        (optimizer, variable, slot variable) relationship; none of the three
        depend on the others directly.

        Note: currently only valid if kind == "user_object".
        """

    @property
    def user_object(self) -> global___SavedUserObject: ...
    @property
    def asset(self) -> global___SavedAsset: ...
    @property
    def function(self) -> global___SavedFunction: ...
    @property
    def variable(self) -> global___SavedVariable: ...
    @property
    def bare_concrete_function(self) -> global___SavedBareConcreteFunction: ...
    @property
    def constant(self) -> global___SavedConstant: ...
    @property
    def resource(self) -> global___SavedResource: ...
    @property
    def captured_tensor(self) -> global___CapturedTensor: ...
    @property
    def saveable_objects(self) -> google.protobuf.internal.containers.MessageMap[builtins.str, global___SaveableObject]:
        """Stores the functions used to save and restore this object. At most one of
        `saveable_objects` or `registered_saver` is defined for each SavedObject.
        See the comment below for the difference between SaveableObject and
        registered savers.
        """

    @property
    def serialized_user_proto(self) -> google.protobuf.any_pb2.Any:
        """The user-generated proto storing metadata for this object, to be passed to
        the registered classes's _deserialize_from_proto method when this object is
        loaded from the SavedModel.
        """

    def __init__(
        self,
        *,
        children: (
            collections.abc.Iterable[
                tensorflow.core.protobuf.trackable_object_graph_pb2.TrackableObjectGraph.TrackableObject.ObjectReference
            ]
            | None
        ) = ...,
        dependencies: (
            collections.abc.Iterable[
                tensorflow.core.protobuf.trackable_object_graph_pb2.TrackableObjectGraph.TrackableObject.ObjectReference
            ]
            | None
        ) = ...,
        slot_variables: (
            collections.abc.Iterable[
                tensorflow.core.protobuf.trackable_object_graph_pb2.TrackableObjectGraph.TrackableObject.SlotVariableReference
            ]
            | None
        ) = ...,
        user_object: global___SavedUserObject | None = ...,
        asset: global___SavedAsset | None = ...,
        function: global___SavedFunction | None = ...,
        variable: global___SavedVariable | None = ...,
        bare_concrete_function: global___SavedBareConcreteFunction | None = ...,
        constant: global___SavedConstant | None = ...,
        resource: global___SavedResource | None = ...,
        captured_tensor: global___CapturedTensor | None = ...,
        saveable_objects: collections.abc.Mapping[builtins.str, global___SaveableObject] | None = ...,
        registered_name: builtins.str | None = ...,
        serialized_user_proto: google.protobuf.any_pb2.Any | None = ...,
        registered_saver: builtins.str | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "asset",
            b"asset",
            "bare_concrete_function",
            b"bare_concrete_function",
            "captured_tensor",
            b"captured_tensor",
            "constant",
            b"constant",
            "function",
            b"function",
            "kind",
            b"kind",
            "resource",
            b"resource",
            "serialized_user_proto",
            b"serialized_user_proto",
            "user_object",
            b"user_object",
            "variable",
            b"variable",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "asset",
            b"asset",
            "bare_concrete_function",
            b"bare_concrete_function",
            "captured_tensor",
            b"captured_tensor",
            "children",
            b"children",
            "constant",
            b"constant",
            "dependencies",
            b"dependencies",
            "function",
            b"function",
            "kind",
            b"kind",
            "registered_name",
            b"registered_name",
            "registered_saver",
            b"registered_saver",
            "resource",
            b"resource",
            "saveable_objects",
            b"saveable_objects",
            "serialized_user_proto",
            b"serialized_user_proto",
            "slot_variables",
            b"slot_variables",
            "user_object",
            b"user_object",
            "variable",
            b"variable",
        ],
    ) -> None: ...
    def WhichOneof(
        self, oneof_group: typing.Literal["kind", b"kind"]
    ) -> (
        typing.Literal[
            "user_object", "asset", "function", "variable", "bare_concrete_function", "constant", "resource", "captured_tensor"
        ]
        | None
    ): ...

global___SavedObject = SavedObject

@typing.final
class SavedUserObject(google.protobuf.message.Message):
    """A SavedUserObject is an object (in the object-oriented language of the
    TensorFlow program) of some user- or framework-defined class other than
    those handled specifically by the other kinds of SavedObjects.

    This object cannot be evaluated as a tensor, and therefore cannot be bound
    to an input of a function.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    IDENTIFIER_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    identifier: builtins.str
    """Corresponds to a registration of the type to use in the loading program."""
    metadata: builtins.str
    """Metadata for deserializing this object.

    Deprecated! At the time of deprecation, Keras was the only user of this
    field, and its saving and loading code will be updated shortly.
    Please save your application-specific metadata to a separate file.
    """
    @property
    def version(self) -> tensorflow.core.framework.versions_pb2.VersionDef:
        """Version information from the producer of this SavedUserObject."""

    def __init__(
        self,
        *,
        identifier: builtins.str | None = ...,
        version: tensorflow.core.framework.versions_pb2.VersionDef | None = ...,
        metadata: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["version", b"version"]) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing.Literal["identifier", b"identifier", "metadata", b"metadata", "version", b"version"]
    ) -> None: ...

global___SavedUserObject = SavedUserObject

@typing.final
class SavedAsset(google.protobuf.message.Message):
    """A SavedAsset points to an asset in the MetaGraph.

    When bound to a function this object evaluates to a tensor with the absolute
    filename. Users should not depend on a particular part of the filename to
    remain stable (e.g. basename could be changed).
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ASSET_FILE_DEF_INDEX_FIELD_NUMBER: builtins.int
    asset_file_def_index: builtins.int
    """Index into `MetaGraphDef.asset_file_def[]` that describes the Asset.

    Only the field `AssetFileDef.filename` is used. Other fields, such as
    `AssetFileDef.tensor_info`, MUST be ignored.
    """
    def __init__(self, *, asset_file_def_index: builtins.int | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["asset_file_def_index", b"asset_file_def_index"]) -> None: ...

global___SavedAsset = SavedAsset

@typing.final
class SavedFunction(google.protobuf.message.Message):
    """A function with multiple signatures, possibly with non-Tensor arguments."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONCRETE_FUNCTIONS_FIELD_NUMBER: builtins.int
    FUNCTION_SPEC_FIELD_NUMBER: builtins.int
    @property
    def concrete_functions(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @property
    def function_spec(self) -> global___FunctionSpec: ...
    def __init__(
        self,
        *,
        concrete_functions: collections.abc.Iterable[builtins.str] | None = ...,
        function_spec: global___FunctionSpec | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["function_spec", b"function_spec"]) -> builtins.bool: ...
    def ClearField(
        self, field_name: typing.Literal["concrete_functions", b"concrete_functions", "function_spec", b"function_spec"]
    ) -> None: ...

global___SavedFunction = SavedFunction

@typing.final
class CapturedTensor(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    NAME_FIELD_NUMBER: builtins.int
    CONCRETE_FUNCTION_FIELD_NUMBER: builtins.int
    name: builtins.str
    """Name of captured tensor"""
    concrete_function: builtins.str
    """Name of concrete function which contains the computed graph tensor."""
    def __init__(self, *, name: builtins.str | None = ..., concrete_function: builtins.str | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["concrete_function", b"concrete_function", "name", b"name"]) -> None: ...

global___CapturedTensor = CapturedTensor

@typing.final
class SavedConcreteFunction(google.protobuf.message.Message):
    """Stores low-level information about a concrete function. Referenced in either
    a SavedFunction or a SavedBareConcreteFunction.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    BOUND_INPUTS_FIELD_NUMBER: builtins.int
    CANONICALIZED_INPUT_SIGNATURE_FIELD_NUMBER: builtins.int
    OUTPUT_SIGNATURE_FIELD_NUMBER: builtins.int
    @property
    def bound_inputs(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]: ...
    @property
    def canonicalized_input_signature(self) -> tensorflow.core.protobuf.struct_pb2.StructuredValue:
        """Input in canonicalized form that was received to create this concrete
        function.
        """

    @property
    def output_signature(self) -> tensorflow.core.protobuf.struct_pb2.StructuredValue:
        """Output that was the return value of this function after replacing all
        Tensors with TensorSpecs. This can be an arbitrary nested function and will
        be used to reconstruct the full structure from pure tensors.
        """

    def __init__(
        self,
        *,
        bound_inputs: collections.abc.Iterable[builtins.int] | None = ...,
        canonicalized_input_signature: tensorflow.core.protobuf.struct_pb2.StructuredValue | None = ...,
        output_signature: tensorflow.core.protobuf.struct_pb2.StructuredValue | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "canonicalized_input_signature", b"canonicalized_input_signature", "output_signature", b"output_signature"
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "bound_inputs",
            b"bound_inputs",
            "canonicalized_input_signature",
            b"canonicalized_input_signature",
            "output_signature",
            b"output_signature",
        ],
    ) -> None: ...

global___SavedConcreteFunction = SavedConcreteFunction

@typing.final
class SavedBareConcreteFunction(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CONCRETE_FUNCTION_NAME_FIELD_NUMBER: builtins.int
    ARGUMENT_KEYWORDS_FIELD_NUMBER: builtins.int
    ALLOWED_POSITIONAL_ARGUMENTS_FIELD_NUMBER: builtins.int
    FUNCTION_SPEC_FIELD_NUMBER: builtins.int
    concrete_function_name: builtins.str
    """Identifies a SavedConcreteFunction."""
    allowed_positional_arguments: builtins.int
    """The prefix of `argument_keywords` which may be identified by position."""
    @property
    def argument_keywords(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]:
        """A sequence of unique strings, one per Tensor argument."""

    @property
    def function_spec(self) -> global___FunctionSpec:
        """The spec of the function that this ConcreteFunction is traced from. This
        allows the ConcreteFunction to be called with nest structure inputs. This
        field may not be populated. If this field is absent, the concrete function
        can only be called with flat inputs.
        TODO(b/169361281): support calling saved ConcreteFunction with structured
        inputs in C++ SavedModel API.
        """

    def __init__(
        self,
        *,
        concrete_function_name: builtins.str | None = ...,
        argument_keywords: collections.abc.Iterable[builtins.str] | None = ...,
        allowed_positional_arguments: builtins.int | None = ...,
        function_spec: global___FunctionSpec | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["function_spec", b"function_spec"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "allowed_positional_arguments",
            b"allowed_positional_arguments",
            "argument_keywords",
            b"argument_keywords",
            "concrete_function_name",
            b"concrete_function_name",
            "function_spec",
            b"function_spec",
        ],
    ) -> None: ...

global___SavedBareConcreteFunction = SavedBareConcreteFunction

@typing.final
class SavedConstant(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    OPERATION_FIELD_NUMBER: builtins.int
    operation: builtins.str
    """An Operation name for a ConstantOp in this SavedObjectGraph's MetaGraph."""
    def __init__(self, *, operation: builtins.str | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["operation", b"operation"]) -> None: ...

global___SavedConstant = SavedConstant

@typing.final
class SavedVariable(google.protobuf.message.Message):
    """Represents a Variable that is initialized by loading the contents from the
    checkpoint.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DTYPE_FIELD_NUMBER: builtins.int
    SHAPE_FIELD_NUMBER: builtins.int
    TRAINABLE_FIELD_NUMBER: builtins.int
    SYNCHRONIZATION_FIELD_NUMBER: builtins.int
    AGGREGATION_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    DEVICE_FIELD_NUMBER: builtins.int
    EXPERIMENTAL_DISTRIBUTED_VARIABLE_COMPONENTS_FIELD_NUMBER: builtins.int
    dtype: tensorflow.core.framework.types_pb2.DataType.ValueType
    trainable: builtins.bool
    synchronization: tensorflow.core.framework.variable_pb2.VariableSynchronization.ValueType
    aggregation: tensorflow.core.framework.variable_pb2.VariableAggregation.ValueType
    name: builtins.str
    device: builtins.str
    @property
    def shape(self) -> tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto: ...
    @property
    def experimental_distributed_variable_components(
        self,
    ) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___SavedVariable]:
        """List of component variables for a distributed variable.

        When this field is non-empty, the SavedVariable will be assumed
        to be a distributed variable defined by the components listed here.

        This is only supported by experimental loaders at the moment.
        """

    def __init__(
        self,
        *,
        dtype: tensorflow.core.framework.types_pb2.DataType.ValueType | None = ...,
        shape: tensorflow.core.framework.tensor_shape_pb2.TensorShapeProto | None = ...,
        trainable: builtins.bool | None = ...,
        synchronization: tensorflow.core.framework.variable_pb2.VariableSynchronization.ValueType | None = ...,
        aggregation: tensorflow.core.framework.variable_pb2.VariableAggregation.ValueType | None = ...,
        name: builtins.str | None = ...,
        device: builtins.str | None = ...,
        experimental_distributed_variable_components: collections.abc.Iterable[global___SavedVariable] | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["shape", b"shape"]) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "aggregation",
            b"aggregation",
            "device",
            b"device",
            "dtype",
            b"dtype",
            "experimental_distributed_variable_components",
            b"experimental_distributed_variable_components",
            "name",
            b"name",
            "shape",
            b"shape",
            "synchronization",
            b"synchronization",
            "trainable",
            b"trainable",
        ],
    ) -> None: ...

global___SavedVariable = SavedVariable

@typing.final
class FunctionSpec(google.protobuf.message.Message):
    """Represents `FunctionSpec` used in `Function`. This represents a
    function that has been wrapped as a TensorFlow `Function`.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _JitCompile:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _JitCompileEnumTypeWrapper(
        google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[FunctionSpec._JitCompile.ValueType], builtins.type
    ):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        DEFAULT: FunctionSpec._JitCompile.ValueType  # 0
        ON: FunctionSpec._JitCompile.ValueType  # 1
        OFF: FunctionSpec._JitCompile.ValueType  # 2

    class JitCompile(_JitCompile, metaclass=_JitCompileEnumTypeWrapper):
        """Whether the function should be compiled by XLA.

        The public interface to `tf.function` uses an optional boolean to
        represent three distinct states for this field.  Unfortunately, proto3
        removes the ability to explicitly check for the presence or absence of a
        field, so we instead map to an enum.

        See `tf.function` for details.
        """

    DEFAULT: FunctionSpec.JitCompile.ValueType  # 0
    ON: FunctionSpec.JitCompile.ValueType  # 1
    OFF: FunctionSpec.JitCompile.ValueType  # 2

    FULLARGSPEC_FIELD_NUMBER: builtins.int
    IS_METHOD_FIELD_NUMBER: builtins.int
    INPUT_SIGNATURE_FIELD_NUMBER: builtins.int
    JIT_COMPILE_FIELD_NUMBER: builtins.int
    is_method: builtins.bool
    """Whether this represents a class method."""
    jit_compile: global___FunctionSpec.JitCompile.ValueType
    @property
    def fullargspec(self) -> tensorflow.core.protobuf.struct_pb2.StructuredValue:
        """Full arg spec from inspect.getfullargspec()."""

    @property
    def input_signature(self) -> tensorflow.core.protobuf.struct_pb2.StructuredValue:
        """The input signature, if specified."""

    def __init__(
        self,
        *,
        fullargspec: tensorflow.core.protobuf.struct_pb2.StructuredValue | None = ...,
        is_method: builtins.bool | None = ...,
        input_signature: tensorflow.core.protobuf.struct_pb2.StructuredValue | None = ...,
        jit_compile: global___FunctionSpec.JitCompile.ValueType | None = ...,
    ) -> None: ...
    def HasField(
        self, field_name: typing.Literal["fullargspec", b"fullargspec", "input_signature", b"input_signature"]
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "fullargspec",
            b"fullargspec",
            "input_signature",
            b"input_signature",
            "is_method",
            b"is_method",
            "jit_compile",
            b"jit_compile",
        ],
    ) -> None: ...

global___FunctionSpec = FunctionSpec

@typing.final
class SavedResource(google.protobuf.message.Message):
    """A SavedResource represents a TF object that holds state during its lifetime.
    An object of this type can have a reference to a:
    create_resource() and an initialize() function.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DEVICE_FIELD_NUMBER: builtins.int
    device: builtins.str
    """A device specification indicating a required placement for the resource
    creation function, e.g. "CPU". An empty string allows the user to select a
    device.
    """
    def __init__(self, *, device: builtins.str | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["device", b"device"]) -> None: ...

global___SavedResource = SavedResource

@typing.final
class SaveableObject(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SAVE_FUNCTION_FIELD_NUMBER: builtins.int
    RESTORE_FUNCTION_FIELD_NUMBER: builtins.int
    save_function: builtins.int
    """Node ids of concrete functions for saving and loading from a checkpoint.
    These functions save and restore directly from tensors.
    """
    restore_function: builtins.int
    def __init__(self, *, save_function: builtins.int | None = ..., restore_function: builtins.int | None = ...) -> None: ...
    def ClearField(
        self, field_name: typing.Literal["restore_function", b"restore_function", "save_function", b"save_function"]
    ) -> None: ...

global___SaveableObject = SaveableObject



import io
from _typeshed import Incomplete, StrPath
from typing import Final
from typing_extensions import TypeAlias

from .cffi_opcode import *
from .error import VerificationError as VerificationError

VERSION_BASE: Final = 9729
VERSION_EMBEDDED: Final = 9985
VERSION_CHAR16CHAR32: Final = 10241
USE_LIMITED_API: Final = True

class GlobalExpr:
    name: Incomplete
    address: Incomplete
    type_op: Incomplete
    size: Incomplete
    check_value: Incomplete
    def __init__(self, name, address, type_op, size: int = 0, check_value: int = 0) -> None: ...
    def as_c_expr(self) -> str: ...
    def as_python_expr(self) -> str: ...

class FieldExpr:
    name: Incomplete
    field_offset: Incomplete
    field_size: Incomplete
    fbitsize: Incomplete
    field_type_op: Incomplete
    def __init__(self, name, field_offset, field_size, fbitsize, field_type_op) -> None: ...
    def as_c_expr(self) -> str: ...
    def as_python_expr(self) -> None: ...
    def as_field_python_expr(self) -> str: ...

class StructUnionExpr:
    name: Incomplete
    type_index: Incomplete
    flags: Incomplete
    size: Incomplete
    alignment: Incomplete
    comment: Incomplete
    first_field_index: Incomplete
    c_fields: Incomplete
    def __init__(self, name, type_index, flags, size, alignment, comment, first_field_index, c_fields) -> None: ...
    def as_c_expr(self) -> str: ...
    def as_python_expr(self) -> str: ...

class EnumExpr:
    name: Incomplete
    type_index: Incomplete
    size: Incomplete
    signed: Incomplete
    allenums: Incomplete
    def __init__(self, name, type_index, size, signed, allenums) -> None: ...
    def as_c_expr(self) -> str: ...
    def as_python_expr(self) -> str: ...

class TypenameExpr:
    name: Incomplete
    type_index: Incomplete
    def __init__(self, name, type_index) -> None: ...
    def as_c_expr(self) -> str: ...
    def as_python_expr(self) -> str: ...

class Recompiler:
    ffi: Incomplete
    module_name: str
    target_is_python: bool
    def __init__(self, ffi, module_name: str, target_is_python: bool = False) -> None: ...
    def needs_version(self, ver: int) -> None: ...
    cffi_types: list[Incomplete]
    def collect_type_table(self) -> None: ...
    ALL_STEPS: list[str]
    def collect_step_tables(self) -> None: ...
    def write_source_to_f(self, f, preamble: str) -> None: ...
    def write_c_source_to_f(self, f, preamble: str) -> None: ...
    def write_py_source_to_f(self, f) -> None: ...

NativeIO: TypeAlias = io.StringIO

def make_c_source(ffi, module_name: str, preamble: str, target_c_file, verbose: bool = False): ...
def make_py_source(ffi, module_name: str, target_py_file, verbose: bool = False): ...
def recompile(
    ffi,
    module_name: str | bytes,
    preamble: str | None,
    tmpdir: str = ".",
    call_c_compiler: bool = True,
    c_file=None,
    source_extension: str = ".c",
    extradir: StrPath | None = None,
    compiler_verbose: int = 1,
    target: str | None = None,
    debug: int | None = None,
    uses_ffiplatform: bool = True,
    **kwds,
): ...



import optparse
from collections.abc import Sequence

def parseOpts(
    overrideArguments: Sequence[str] | None = None, ignore_config_files: str = "if_override"
) -> tuple[_YoutubeDLOptionParser, optparse.Values, list[str]]: ...

class _YoutubeDLOptionParser(optparse.OptionParser):
    ALIAS_DEST: str
    ALIAS_TRIGGER_LIMIT: int
    def __init__(self) -> None: ...
    def parse_known_args(
        self, args: Sequence[str] | None = None, values: optparse.Values | None = None, strict: bool = True
    ) -> tuple[optparse.Values, list[str]]: ...

def create_parser() -> _YoutubeDLOptionParser: ...



from .resource import Collection, Model

class Config(Model):
    id_attribute: str
    @property
    def name(self) -> str: ...
    def remove(self) -> bool: ...

class ConfigCollection(Collection[Config]):
    model: type[Config]
    def create(self, **kwargs) -> Config: ...  # type:ignore[override]
    def get(self, config_id: str) -> Config: ...
    def list(self, **kwargs) -> list[Config]: ...



from collections.abc import Callable, Hashable
from typing import Literal
from typing_extensions import TypeAlias

from networkx.classes.graph import Graph, _Node

__all__ = ["greedy_source_expansion"]

_Algorithm: TypeAlias = Literal["clauset"]

ALGORITHMS: dict[_Algorithm, Callable[[Graph[Hashable], Hashable, int | None], set[Hashable]]]

def greedy_source_expansion(
    G: Graph[_Node], *, source: _Node, cutoff: int | None = None, method: _Algorithm = "clauset"
) -> set[_Node | None]: ...



from authlib.oauth2.rfc6749 import TokenEndpoint

class RevocationEndpoint(TokenEndpoint):
    ENDPOINT_NAME: str
    def authenticate_token(self, request, client): ...
    def check_params(self, request, client) -> None: ...
    def create_endpoint_response(self, request): ...
    def query_token(self, token_string, token_type_hint) -> None: ...
    def revoke_token(self, token, request) -> None: ...



from typing import Final

__docformat__: Final = "reStructuredText"
document_template: Final[str]

def blahtexml(math_code: str, as_block: bool = False) -> str: ...
def latexml(math_code: str, as_block: bool = False) -> str: ...
def pandoc(math_code: str, as_block: bool = False) -> str: ...
def ttm(math_code: str, as_block: bool = False) -> str: ...



from _typeshed import Incomplete
from typing import ClassVar
from typing_extensions import deprecated

from setuptools import Command
from setuptools.warnings import SetuptoolsDeprecationWarning

class develop(Command):
    user_options: ClassVar[list[tuple[str, str | None, str]]]
    boolean_options: ClassVar[list[str]]
    install_dir: Incomplete
    no_deps: bool
    user: bool
    prefix: Incomplete
    index_url: Incomplete
    def run(self) -> None: ...
    @deprecated(
        "develop command is deprecated. Please avoid running `setup.py` and `develop`. "
        "Instead, use standards-based tools like pip or uv."
    )
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...

class DevelopDeprecationWarning(SetuptoolsDeprecationWarning): ...



from ...protocol.rfc4511 import LDAPDN
from ..operation import ExtendedOperation

class PartitionEntryCount(ExtendedOperation):
    request_name: str
    response_name: str
    request_value: LDAPDN
    response_attribute: str
    def config(self) -> None: ...
    def __init__(self, connection, partition_dn, controls=None) -> None: ...
    def populate_result(self) -> None: ...
